﻿using System.Collections.Generic;
using System.IO;

namespace SessionModManagerCore.Classes
{
    public class TexturePathInfo
    {
        public string TextureName { get; set; }
        public string RelativePath { get; set; }

        /// <summary>
        /// Initializes list of known texture/meshes and their relative path within the game
        /// </summary>
        /// <remarks>
        /// This code below is generated by running the Utils\getTexturePaths.ps1 script. It uses UnrealPak to get the list of files in Customization folder.
        /// </remarks>
        public static List<TexturePathInfo> InitTexturePaths()
        {
            List<TexturePathInfo>  _texturePaths = new List<TexturePathInfo>();

            _texturePaths.Add(new TexturePathInfo() { TextureName = "SKXX_Customization_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Skelmesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_48mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_149_Mid_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Default_Blank850_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ESP_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "GridItem_HatImage", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "GridItem_PantsImage", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_REWARD_ALL", RelativePath = $"Customization" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_ZeroNumero", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_ZeroBlood", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_SingleSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_Spiral", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_SingleBloodSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_PunkStarPink", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_Painkiller", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_LegacyRansom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_DamnItAllVxFire", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_DamnItAllFont", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_ClassicBold", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_BloodSkullGold", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_BloodSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_American", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_3SkullBlood", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_Heroin_TShirt_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_Heroin_TShirt_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Fallen_Trademark", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Fallen_Stripes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Fallen_RiseWith", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Fallen_Revolver", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Fallen_NightFiend", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Fallen_Insignia", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Fallen_Hendrix", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Fallen_Desert", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Fallen_DeadHead", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_ESP_OgGraphic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_MidTopCup", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_VeryBigEgg_Custom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_ToxicEyesFade", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_ToxicEye_CustomShape", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_ToxicEye_825", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_TomDay", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_TheEggxorcist", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_TheEgg_Custom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_Spiral_Custom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_RazorEgg_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_RazorEgg", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_GreenEye", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_HighScores", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_DMODW", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_DeerMan", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_CurbCrusher", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_CraigScott_Player1", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_CraigScott", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_BlackEye_Custom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_BigEgg_Yellow_Custom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_BigEgg_Pruple_Custom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_Anatomy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_StinkerPudwill_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_AnimalPudwill_7900", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_StoneagePudwill_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_BluePlanetPudwill_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_TorusUltra_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_TorusUltra_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_TorusUltra_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_StreetSlims_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_StreetSlims_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_GT1Wideboy_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_GT1Wideboy_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_Evolution_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_DualUltraClassic_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_DualUltraClassic_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_DualUltraClassic_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_ABXTorus_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_ABXTorus_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_ABS_Nexus_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_ABS_Nexus_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_ABS_Nexus_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_ABS_Nexus_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_Evolution_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Autobahn_Evolution_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_ClassicPandaSamarriaRhodamine_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_JazzNightSamaria_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_RThompson_B_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_RThompson_A_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_RThompson_C_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_JahPandaSamarria_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_TeddyLavenderDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_MagildaLavenderTealStain", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Element_Landrein", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Element_MoonDust", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Element_Square30", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_FairyTaleMiniTealBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_FairyTailPink", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_TeddyPrism", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jart_Manny_Skull_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_HOOD_Zero_Ransom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Roger_WeedAndCobra", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LB_GEN_Cargo_Denim", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LB_GEN_Cargo_Camo_Denim", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_Fallen_Bomber_RWTF", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_Fallen_Bomber", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_ClassicPandaBarletta_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_PeekabooProPandaBarletta_9500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_JazzNightBarletta_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Antilanta_EnterTheMatrixGlow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_DaneBurman", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jart_Manny_Flag_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_Howlin_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_StoneageDaewon_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_CoolerDaewon_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_BalancedDaewon_BC_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_Revenge_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_DaewonRampage_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_BurmanPastForms", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_BurmanDevilleSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_DaneBurman_Incest", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_SharkSnackPudwill_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_BurmanDeadPresidents", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_RP_3_8", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}RiserPads{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_RP_1_8", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}RiserPads{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_RP_1_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}RiserPads{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_RP_1_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}RiserPads{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_RP_1_16", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}RiserPads{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Antilanta_Voorhees", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Antilanta_TimboStomp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Antilanta_ChessOverCheckers", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_StokedDivision_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Magenta_VX", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Magenta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Magenta_Camera", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Magenta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Magenta_Bear", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Magenta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Flypaper_Diecut", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Flypaper{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Orange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PBP_ColorSelector_Widget", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "ColorSpectrum", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "ColorPickerSliderCursor", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "ColorPickerCursor", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "M_ColorPicker_Slider", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PBP_CustomizationGrid_Widget", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PBP_CustomizationGridItem_Widget", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PBP_BrandFilterWidget", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "GridItem_NoneImage", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PBP_BrandFilterItemWidget", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "DATASS_DatabaseCustomization", RelativePath = $"Customization" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "DATASS_DatabaseCompanies", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "DATASS_DatabaseCategories", RelativePath = $"Customization" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "DATASS_CITContactPartsDecayDatabase", RelativePath = $"Customization" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "DATASS_CITClothesContactPartsDecay", RelativePath = $"Customization" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LB_GEN_StriaghtCut_Denim", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LB_GEN_StriaghtCut_Cotton", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LB_GEN_Jeans_Denim", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LB_GEN_Jeans_Camo_Denim", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LB_GEN_Jeans_Cotton", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_Head_Cap_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_Head_Beanie", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_UB_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_TShirt_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_TShirt_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_TShirt_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_Jacket_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_Jacket_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_Jacket_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_Hoodie_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_Hoodie_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_Hoodie_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_Hoodie_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_Hoodie_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_Hoodie_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_Hoodie_00", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_LB_NoComply_Cargo_Pants_A_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_LB_NoComply_Cargo_Pants_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_HD_Hair_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_HD_Hair_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_Shoes_Vul_A_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_Shoes_Vul_A_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_Shoes_Vul_A_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_Shoes_Vul_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_UB_TankTop_A_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_UB_TankTop_A_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_UB_TankTop_A_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_Shoes_Vul_A_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_UB_TankTop_A_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_Shoes_Vul_A_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_UB_TankTop_A_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_UB_TankTop_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_UB_Shirt_A_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_UB_Shirt_A_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_UB_Shirt_A_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_UB_Shirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_A_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_A_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_HD_Hat_No_Hair_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_HD_Hat_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_HD_Hair_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_HD_Hair_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_HD_Hair_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_HD_Hair_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_HD_Hat_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ZeroSkateboards_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Welcome", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_SugarSkateCo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}SugarSkateCO{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_SucksClothing", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Snot", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_SML_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CusLogo_SchlaudieSkateCo_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Schlaudie{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_SatoriWheels_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_SantaCruz", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}SantaCruzSkateboards{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CusLogo_RogerSkateCo_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Ricta", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}RictaWheelDynamics{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_OJs", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}OJWheels{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_NoComply_CustomizationLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_MOB_Griptape", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}MOBGrip{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_MeowSkateboards", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_MagentaSkateboards", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_MadridSkateboards", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_MadridFlypaper", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Krux", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Jenkem_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}JenkemMag{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Jart", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_iDabble", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}iDabble" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_HeroinSkateboard_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_HazeWheels", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HazeWheels" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GrindKing_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Globe_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Globe{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Flip_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_FlatFace_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Fallen_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Fallen{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Faded", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Faded" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_eS_Footwear_RED_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Stickers{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_ThankYouSkateboards", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}ThankYouSkateboards" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Enjoi", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Destructo_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_DGK", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Element", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Brandalised", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_AutobahnWheelCo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Autobahn{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Antilanta", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Adidas_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Adidas{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Translucent_RadialWide_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_Wide_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_Wide_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_Wide_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_Wide_51mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_Wide_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_Wide_48mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_Translucent_Wide_Conic_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_Translucent_Wide_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_Translucent_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_RadialWide_56mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_RadialWide_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_RadialWide_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_RadialWide_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_RadialWide_48mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_RadialWide_46mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_ConicWide_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_ConicWide_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_ConicWide_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_ConicWide_51mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_ConicWide_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_Wide_46mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_ConicWide_46mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_ConicWide_48mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_46mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_36mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_32mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Defualt_Classic_51mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Default_TortoiseSeries_Wide_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Default_TortoiseSeries_RadialWide_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Default_TortoiseSeries_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Snot_Team_58", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Snot_Team_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Snot_Team_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Default_TortoiseSeries_Wide_Conic_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Snot_Team_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Snot_Team_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Snot_Team_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Snot_LilBoogers_48", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Snot_Team_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_Toonies_YounessAmraniOG_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_Toonies_TomKnox_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_Toonies_MikeArnold_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_TheLoveSeries_TomKnox_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_TheLoveSeries_SammyMontano_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_TheLoveSeries_MikeArnold_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_Team_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_SucculentCruiser_Primes_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_SucculentCruiser_DesertBlues_56mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_SoldticeSeries_YounessAmrani_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_SoldticeSeries_TomKnox_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_SoldticeSeries_AustynGillette_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_ReallysmlWheels_40mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_PencilPushersSeries_TomKnox_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_PencilPushersSeries_JoshPall_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_PencilPushersSeries_ChrisJones_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_PencilPushersSeries_AustynGillette_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_YounessAmrani_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_MikeArnold_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_Mark_Suciu_51mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_GroceryBag_56mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_MaxGeronzi_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_GroceryBag_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_GroceryBag_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_GroceryBag_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_GroceryBag_55mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_DelicateSeries_SammyMontano_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_DelicateSeries_MarkSuciu_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_DelicateSeries_AusytnGilette_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_CoffeeCruiser_56mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_CoffeeCruiser_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_GroceryBag_51mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_CoffeeCruiser_50mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_ClassicSeries_RaymondMolinar_Polaroid_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_ClassicSeries_PianoHands_51mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_ClassicSeries_JamesCraigOG_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_ClassicSeries_DrakeJones_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_CoffeeCruiser_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_BubbsSeries_MarkSuciu_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_BubbsSeries_AustynGillette_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_BackToThe90s_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_AustynGilette_53mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Satori_TopShelfElephant_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Satori_TopShelfElephant_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Satori_TopShelfElephant_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Satori_TopShelfElephant_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Satori_RedEye_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Satori_Paisley_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Satori_MeditationSeries_Classic_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Satori_LinkedLogoTwoTone_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Satori_LilNuggs_GooBall_54mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Satori_DeLaSatori_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Satori_BigLinkSeries_Classic_51mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Ricta_Speedrings_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Ricta_Sparx_MainFrame_Black_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Ricta_Naturals_Vortex_DesArmo_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Ricta_Impeller_Brevard_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SML_ClassicSeries_AusytnsAlmonds_52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Ricta_Clouds_Red_57", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Ricta_Clouds_Black_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_OJ_NoraVasconcellosDreams_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_Twirl_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_Sneak_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_PrimeCut_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_PrimeCut_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_PrimeCut_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_Lurk_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_Lurk_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_Hazzy_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_DeathOnAcid_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_DeathOnAcid_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_DeathOnAcid_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_DeathOnAcid_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_10thYear_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_101PrimeCut_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Haze_101ChiChi_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_169_Mid_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_169_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_159_Mid_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_159_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_149_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_144_Mid_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_144_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_139_Mid_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_139_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_129_Mid_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_129_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Spark_149_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Spark_144_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Kre-per_149_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Kre-per_144_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_GK6_144_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Disruptor_149_Mid_A_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Disruptor_149_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Disruptor_144_Mid_A_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Disruptor_144_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Disruptor_139_Mid_A_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Disruptor_139_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Sendai_144", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Sendai_139", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_D1Raw_149_Mid_A_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_D1Raw_144_Mid_A_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_D1Raw_144_Mid_A_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_D1Raw_144_Mid_A_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_D1Raw_144_Mid_A_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_D1Raw_139_Mid_A_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_D1Raw_139_Mid_A_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_D1Raw_139_Mid_A_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_D1Raw_139_Mid_A_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_D1Avalanche_139_Low_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_D1Avalanche_129_Low_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_NoComply_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_MOB_SmallLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_MOB_RainbowWallpaper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_MOB_RainbowBigLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_MOB_Rainbow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_MOB_PinkStripes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_MOB_BigLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_MOB_ArgyleBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Magenta_Swans", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Magenta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Magenta_Fly", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Magenta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Flypaper_Swarm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Flypaper{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Yellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Tape_Painted", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Tape_Coutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_StokedDivision_Painted", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Spiral_Painted", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Spiral_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Session_Painted", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Session_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Red", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_QuadLines_Painted", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_QuadLines_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Purple", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_LineVertical_Painted", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_LineVertical_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_LineHorizontal_Painted", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_LineHorizontal_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Green", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Geometric_Painted", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Geometric_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_DoubleLine_Painted", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_DoubleLine_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_BracketA_Painted", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_BracketA_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Blue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_BigHorizontal_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Abstract_Painted", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Default_Abstract_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Creature_Bars_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Creature_Barbwire_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Default_UncutBoard", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Default_Blank875_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Default_Blank8250_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Default_Blank8125_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Default_Blank800_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Default_Blank775_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Default_Blank750_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Default_Blank725_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DR_Thin", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DR_Thick", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DR_Bulge", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DR_Bone", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DL_Default_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_BurmanMummy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_BurmanDickman", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_TwentyEyesWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_SoilTealDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_SoilBone", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_SlothBlackLavender", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_PeregrineTealDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_PeregrinePrism", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_PeregrineNeonYellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_PeregrineBlackPurple", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_LooDoodWhiteDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_LooDoodBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_LambyOnAtheme", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_GatewayTealDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_YellowLogo_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_SpotOnTeal_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_SpotOnRed_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_SayCheese_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_RedPlanet_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_RampagePudwill_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_PinkLogo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_OrangeLogo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_NightCloud_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_Medallion_BC_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_JunkFood_8380", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_HealthNut_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_HeadInTheClouds_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_GoodClouds_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_FlameOn_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_EyeSpy_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Ricta_Clouds_White_60", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_Essentials_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_BadClouds_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_ColourBurst_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SugarSkateCo_TypeScript", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SugarSkateCo_HappyCloud", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SugarSkateCo_ZanderMonster", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SugarSkateCo_GinoGonzReissue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SugarSkateCo_Crayon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SugarSkateCo_Brain", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SugarSkateCo_BeachBall", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SugarSkateCo_80sChrome", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Suck_Spoon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SugarSkateCo_GoodVibes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Suck_AfterSex", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Suck_LadySmoker", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Session_CFTW", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Session{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Schlaudie_TentaclesPurple", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Schlaudie_TentaclesGreen", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Schlaudie_Spryte", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Schlaudie_ShipBattle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Session_CIAH", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Session{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Schlaudie_PipStreet", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Schlaudie_MovingCircus", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Schlaudie_Military", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Schlaudie_Kraken", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Schlaudie_Constantine", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Schlaudie_NiteOwl", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Schlaudie_AllRed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SantaCruz_ScreamingHand_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SantaCruz_SalbaTiger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Schlaudie_BirdEyeView", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SantaCruz_KnoxFirepit", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SantaCruz_MeekOGSlasher", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SantaCruz_DressenRoses", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SantaCruz_ClassicDot_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_WhatIWant_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_WeedCobra_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_Wallpaper_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_SwordSwallower_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_SockMonkey_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_SkateSwitch_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_SkateRats_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_ScooterRoach_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_Scent_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_SausageParty_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_ReeseMagnoliaTree_8380", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_Reading_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_PinkStarfish_8380", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_PinkLimo_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_MeSoHorny_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_MaxTaylor_C_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_MaxTaylor_B_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_MaxTaylor_A_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SantaCruz_KendallEndOfTheWorld", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_LangstonNeverMind_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_HalfRoger_C_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_HalfRoger_B_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_HalfRoger_D_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_GhostBoner_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_FreeTheHand_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_FirstPlace_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_Doodler_B_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_Doodler_A_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_DogBalls_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_Bender_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NoComply_Custom_Orange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NoComply_Custom_Magenta", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NoComply_Custom_Green", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_HalfRoger_A_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NoComply_Custom_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NoComply_Custom_Blue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_Precarious_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_OrigamiCat_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_Milo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_MarmaladeTiger_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_Logo_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_Jazz_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_HalleysComet_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_Grandma_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_Flamingo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_Catzilla_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_CatMeowntain_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_BigCat_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_AnnieGugliaWallieTunes_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Magenta_WoodPlantTeam", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Magenta_Station", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Magenta_Medley", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Magenta_BrushTeam", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Magenta_BigPlantTeam", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_WingsYellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_WingsPink", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_WingsMint", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_WingsBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_TaroCardSeries_Reaper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_TaroCardSeries_Lovers", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_MAI_DG_Madrid_HeshlordDezminLane_HeshlordShape", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_MAI_DG_Madrid_HeshlordDezminLane", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_ExplosionYellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_ExplosionWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_ExplosionRed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_ExplosionBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Madrid_DanielYeager", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jart_Woodstock_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jart_Texture_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jart_Golden_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jart_Frame_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jart_Flagship_8375", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_SpaceTiger_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jart_Akbar_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_TeamStrobeOrange_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_TeamStrobeBlue_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_TeamSorryYellow_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_TeamSorryBlack_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_ScriptBlue_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_ScriptBlack_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jart_Classic_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jart_Sculptural_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_OdsseyStencilYellow_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_OdsseyStencilRed_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_BubbleLogoOrange_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_BubbleLogoBlue_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_FlatFace_OldFaces", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_FlatFace_Font", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_FlatFace_FlatFaceGreenFace", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Faded_Firday_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Faded{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Faded_BYOB_8501", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Faded{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_QuattroOdysseyLumoBlack_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_QuattroOdysseyLumoBlue_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_WhityPandaLogoWide_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_SpectrumWhite_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_RastaPanda_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_RainbowFartMint_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_RainbowFartBlack_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_PlaidPandaSamaria_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_KittenDreams_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_HappyTree_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_Funstick_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_Flipper_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_DontFit_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_ClassicPandaSamarriaBlack_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_CatCollage_7000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_AllCapsWhite_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_AllCapsBlack_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Element_Seal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_RenaissanceBarletta_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Enjoi_SpectrumBlack_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Element_MagmaSeal_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Element_AcAppleyardBbs_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_YingYang_8010", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_WetPaint_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Viper_8380", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Tsunami_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Sunshine_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_SlapUp_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_RainbowDrip_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Primo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Monogram_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Element_DaydreamSection_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Element_Blazin_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Guadalupe_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_LoSide_8010", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_GoodLuckSantos_8010", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Friends_8025", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Aztek_8010", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Creature_Police", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Creature_Barbwire", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Bloom_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Connect_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Brandalised_What", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Brandalised_GraffitiIsACrime", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Brandalised_PinkGuardDog", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Brandalised_PandaGuns", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Brandalised_GirlFloatingAway", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Brandalised_FollowYourDreams", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Brandalised_FacemaskRatWithKnife", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Brandalised_BunnyThug", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Antilanta_EnterTheMatrix", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Antilanta_AntixFrko", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Antilanta_AntiLeaf", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_NONE", RelativePath = $"Customization" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SessionOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_ToreyPudwill_Pants", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}ToreyPudwill" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_SessionOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SessionOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_RyanThompson_Hair", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}RyanThompson" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_Manny_Tee", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Manny" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_LouieBarletta_Pants", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}LouieBarletta" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_NBS_Classic52mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Jessie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jesse_760_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Jessie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_JahmirBrown_Pants", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}JahmirBrown" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_JahmirBrown_Hair", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}JahmirBrown" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Zero_Stardust", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Zero_Skull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Zero_Ransom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Zero_DamnItAllVX", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Zero_DamnItAll", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Zero_Army", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_SWT_Zero_ArmyTeam", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_HOOD_Zero_EyeBall", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_HOOD_Zero_DTL", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Sugar_TieDye", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SugarSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Sugar_Gradient", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SugarSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Sugar_Collegiate", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SugarSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Schlaudie_Spryte", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SchlaudieSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Schlaudie_PirateShip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SchlaudieSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Schlaudie_Pip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SchlaudieSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Schlaudie_LilLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SchlaudieSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Satori_Tea_Logo_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Satori" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Satori_Tea_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Satori" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Satori_Rising_Sun", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Satori" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Satori_Classic_Elephant", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Satori" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Roger_SwordSwallower", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Roger_SausageParty_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Roger_PartyYourBunsOff", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Roger_Doodler", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_LongSleeve_Krux_KruxRainbow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_SWT_iDabble_80s", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}iDabble" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Fallen_TradeMark2", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Fallen_TradeMark", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Fallen_RiseWith", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Fallen_Insignia", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Fallen_Graphic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Faded_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Faded" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_SWT_Faded_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Faded" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Destructo_Star", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Destructo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Destructo_Iron", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Destructo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Destructo_Circle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Destructo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Customizable", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Creature_Prison", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Creature_Blank", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TANK_Creature_Blank", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_SessionOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Brandalised_Rat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Brandalised_DreamBig", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Brandalised_Bunny", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Brandalised_What", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Brandalised_Guard", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Brandalised_FollowYourDreams", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Brandalised_Crime", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_LB_Prison_Pants", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Prison" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LB_GEN_CustomStriaghtCut_Denim", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LB_GEN_BaggyPants_Denim", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LB_GEN_BaggyPants_Cotton", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LB_GEN_BaggyPants_Camo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LB_GEN_BaggyPants_Denim", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LB_GEN_BaggyPants_Cotton", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LB_GEN_BaggyPants_Camo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_HD_CAP_Fallen_Trademark", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_HD_CAP_Fallen_Insigina", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_HD_CAP_Fallen_HendrixPatch", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_HD_CAP_Fallen_Claw", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_HD_BEAN_Fallen_Insignia", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_Head_Cap_Destructo_Circle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Destructo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_HD_BEAN_Destructo_Trifecta", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Destructo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_Head_Cap_Customizable", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_Head_Creature_Beanie_Prison", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_Fallen_Trooper_ChrisCole", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_Fallen_TheEasy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_Fallen_Patriot_Marks", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_Fallen_Patriot", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Creature_Shoe_Prison", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_MidTop_Customization", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_NoComply_TShirt_A_10", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_NoComply_TShirt_A_08", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_NoComply_TShirt_A_07", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_NoComply_TShirt_A_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_NoComply_TShirt_A_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_Manny_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Manny" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_NoComply_TShirt_A_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_NoComply_TShirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_Jenkem_TShirt_A_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_Jenkem_TShirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_FlatFace_TShirt_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_FlatFace_TShirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_FlatFace_TShirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_FlatFace_TShirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_FlatFace_Hoodie_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_FlatFace_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_Jenkem_Cap_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_Jenkem_Cap_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_Jenkem_Cap_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_Shoes_Vul_B_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_Shoes_Vul_B_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_Shoes_Vul_B_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_Shoes_Vul_B_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_Shoes_Vul_B_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_Globe_Mahalo01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Globe{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_eSFootwear_Sal23_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_eSFootwear_Sal23_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_eSFootwear_Sal23_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_eSFootwear_Sal23_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_eSFootwear_Explorer_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_eSFootwear_AccelOG_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_eSFootwear_AccelOG_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_eSFootwear_AccelOG_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_eSFootwear_AccelOG_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_NoComply_TShirt_A_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_eSFootwear_AccelOG_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "ABP_CustomizationCharacter", RelativePath = $"Customization" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_FullBody_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_FT_eSFootwear_AccelOG_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_NoComply_TShirt_A_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Body", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_EyeRefractive", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_Hair", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Head", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Body_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Body_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Body_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Head_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Head_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Head_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_PumpkinMan", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_FullBody_Male04_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_FullBody_Male03_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_FullBody_Male02_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Denim_Gen_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Denim_Gen_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_WheelEmpty", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "pumpkin_material_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_HD_BEANIE_Creature_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Male02_Face", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Male02_Body", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_EyeRefractive_Male02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_FullBody_Female04_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_FullBody_Female03_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_FullBody_Female02_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_FullBody_Female_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Male03_Head", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Male03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Male03_Eyes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Male03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Male03_Eyelashes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Male03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Male03_Body", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Male03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Patriot_Marks_SandBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Male04_Head", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Male04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Male04_Eyes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Male04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Male04_Body", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Male04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DenimThreadPattern_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fabric" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DenimFabric", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fabric" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_StraightCut_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_StraightCut_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Denim_Gen_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "umonab2dy_4K_Albedo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fabric" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Cap_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GEN_HD_CAP_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_WindMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Cap_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Master" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Male_TShirt_GEN_Mask_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}BodyMasks" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CupSole_Shoes_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shoes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CupSole_Shoes_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shoes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Cargo_Pants_03_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Cargo_Pants_02_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Faded", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Faded" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GEN_ClothDirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_v2_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Reworked" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Cloth_UB_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Master" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GEN_HD_CAP_Masks_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Tuque_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Tuque_ID", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Tuque_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_Stencil_Creature", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "New_Graph_embrotest", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_HD_BEANIE_Creature_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Male02_Face_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Male02_Face_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Male02_Face_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Male02_Body_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Male02_Body_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "pumpkin_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Shoes_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Shoes_ID", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Shoes_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Female04_Head", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Female04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Female04_FacialHair", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Female04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Female04_Eyes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Female04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Female04_Body", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Female04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_StraightCut_02_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Eyelashes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Female03_Head", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Female03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Female03_Eyes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Female03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_EyeOcclusion_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Female03_Body", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Female03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_EyeRefractive", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_EyeOcclusion", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Body_Underwear", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Head", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_EyeRefractive_Female02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Head_Female02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Body_Female02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_StraightCut_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassic_Default_52_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_8125_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_BasePlate_Default_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_RiserPad_1_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}RiserPads{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8125_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Male03_Head_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Male03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Male03_Head_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Male03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Male03_Head_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Male03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_FEET_Fallen_Atlas_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_FEET_Fallen_Atlas_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_FEET_Fallen_Atlas_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Patriot_Marks_Forest", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ClotColorMask_Atlas_2x2", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Male03_Body_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Male03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Male03_Body_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Male03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Male03_Body_BC_ReducedBlemish", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Male03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Hair_AL", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Male04_Head_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Male04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Male04_Head_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Male04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Male04_Head_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Male04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Male04_Body_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Male04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Male04_Body_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Male04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_StraightCut_ColorIDMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "M_Apparel_LowerBody_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Master" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MF_SampleSelector", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Functions" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MF_HighlightSelected", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Functions" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MF_ColorHandle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Functions" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MF_ClothCamo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Functions" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MF_ClothUVTranforms", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Functions" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Female04_Head_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Female04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Female04_Head_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Female04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Female04_Head_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Female04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Female03_Head_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Female03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Female03_Head_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Female03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Female03_Head_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Female03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Body_Underwear_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Female04_Body_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Female04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_EyeLashes_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Female04_Body_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Female04" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Female02_Hairs_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Female03_Body_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Female03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Female03_Body_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Female03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Body_Underwear_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Body_Underwear_BC_BlackUw", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Head_Female02_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Head_Female02_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Head_Female02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Body_Female02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Body_Female02_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Head_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Head_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Head_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_HG_Default_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WheelBase_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_BR_Default_A_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bearings{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BP_Default_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BG_Default_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bushing{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BP_Default_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Raw", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BASE" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DL_DeckLayers_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Raw", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_NT_Default_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Nuts{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_MASTER_Shoe_Atlas", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Master" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Hanger_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Hanger_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Hanger_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Baseplate_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Baseplate_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Baseplate_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_Default_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_Default_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_Default_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BG_Default_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bushing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BG_Default_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bushing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BG_Default_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bushing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Default_Dirt_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Default_Dirt_01_Alpha_v2", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Default_02_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "New_Graph_outputWheeltest", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BR_Default_A_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bearings{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BR_Default_A_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bearings{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Wheel_Base_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BR_Default_A_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bearings{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WD_Scratch_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WD_Default_Scratch_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WD_Default_Clean_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WD_Default_Scratch_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_Default_Scratch", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_Designs_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_Damage_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Griptape_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Griptape_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MASTER" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DL_Default_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DL_Default_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_NT_Default_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Nuts{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_NT_Default_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Nuts{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_NT_Default_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Nuts{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DL_Default_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassic_Default_54_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_129_Mid_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_GrindKing_Spark__144", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_HUB_JenkemBook_Vol1", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}JenkemMag{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}StaticMeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_BasePlate_GrindKing_Spark", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Green", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Red", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Purple", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Transparent_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Orange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Truck_GrindKing_Spark_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Truck_GrindKing_Spark_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_JenkemBooks", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}JenkemMag{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Wheel_Transparent", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Jenkem_Books", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}JenkemMag{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_GrindKing_Spark_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_GrindKing_Spark_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_GrindKing_Spark_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_GrindKing_Spark_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_GrindKing_Spark_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_GrindKing_Spark_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PBP_EditSkaterName_Widget", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PBP_CharacterCustomization", RelativePath = $"Customization" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PBP_CustomizationSkateboard", RelativePath = $"Customization" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PBP_CustomizationCharacter", RelativePath = $"Customization" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Rail_Default_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DeckRail_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_DeckRail_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Denim_Gen_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_sRGB_Antilanta_Padded", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Antilanta" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Hair_AL", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_PlaceHolder", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_BC_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_BaseMesh_BoxArtShirt_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_BC_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_ShoesLightGrey_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_Vulc_Shoes_2_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Placeholder_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_HD_BEANIE_Creature_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ChinoFabric", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fabric" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Chino_Tread_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fabric" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Cargo_Pants_01_RG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Cargo_Pants_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Cargo_Pants_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Cargo_Pants_01_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Cap_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_9", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_11", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_12", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_L", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Male_Hoodie_Mask_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}ClothingMask" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_UB_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Male_StraightCut_GEN_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}BodyMasks" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Cotton_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Cotton_Gen_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Cotton_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Cotton_Gen_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Cotton_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Cotton_Gen_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Cotton_ICON_9", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Feet_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Generic{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_GEN_Vulc_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Black_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_149_Mid_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_48mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_48", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Red", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Orange_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Default_Blank850_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassic_Default_48_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_BasePlate_Default_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_149_Mid_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_Shoes_Vul_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Generic{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Male_VulcA_GEN_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}BodyMasks" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_Vul_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Cotton_Gen_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Cotton_ICON_8", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Cotton_ICON_7", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Cotton_Gen_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Cotton_Gen_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Cotton_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Cotton_Gen_K", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_FT_Shoes_Vul_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Cotton_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Cotton_Gen_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Cotton_Gen_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Cotton_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Cotton_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_LB_StraightCut_Pants_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Cotton_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Hoodie_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_UB_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Cotton_Gen_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Hoodie_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Cotton_Gen_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_Hoodie_Mask_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_M", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_14", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_K", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_13", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_10", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_8", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_Camo_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_Camo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_Camo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_7", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Creature_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_HD_Cap_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_A_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Shoes_Gloss", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Shoes_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Default_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_48mm_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_Dirt_0_Alpha", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_WheelOpaque_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BG_Default_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bushing{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_Vulc_Shoes_2_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_Black_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GEN_HD_CAP_Masks_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Session_Hoodie_Heather_Grey_D_withAlpha", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Bushing_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bushing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Bushing_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bushing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Bushing_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bushing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_RiserPad_3_8", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}RiserPads{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_RiserPad_1_8", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}RiserPads{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_RiserPad_1_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}RiserPads{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_RiserPad_1_16", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}RiserPads{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Rail_Default_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Rail_Default_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Rail_Default_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_HUB_JenkemBook_Vol2", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}JenkemMag{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}StaticMeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_BRN_eS_Square3_ShoeProp_R", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Staticmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_BRN_eS_Accel_ShoeBox_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Staticmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Insignia_BlackWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Zero_Army_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Satori_Calssic_Elephant_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Satori" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Roger_WeedAndCobra", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Roger_SwordSwallower", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_HD_BEANIE_Creature_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_HD_BEANIE_Creature_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_HD_BEANIE_Creature_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_SamuraiPunk_TShirt_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_SamuraiPunk_TShirt_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_NoComply_TShirt_08", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_NoComply_TShirt_07", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_NoComply_TShirt_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_NoComply_TShirt_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_NoComply_TShirt_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_NoComply_TShirt_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_NoComply_TShirt_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_NoComply_TShirt_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Heroin_TShirt_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_R_Red", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_R_Navy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_R_Green", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_R_Charcoal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_R_Brown", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_R_Bone", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_R_BlackWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_R_BlackLeather", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_R_BlackGum", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_R_BlackGrey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ShirtJenkem_White", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_BRN_eS_ShoesBox_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Fallen_Atlas_A_Stencil", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_Stencil_ZERO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Zero_Army_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_sRGB_Satori_Padded", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Satori" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_sRGB_RogerSkateCO_Padded", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_Jenkem_TShirtBlack_01_RN", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_Jenkem_TShirt_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_SamuraiPunk_TShirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_ComplyShirt_White_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_ComplyShirt_Pink_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_NoComply_TShirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_ComplyShirt_Green_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_ComplyShirt_BrightYellow_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_ComplyShirt_Blue_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_ComplyShirt_Black_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Heroin_TShirt_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_Heroin_TShirt_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_ComplyShirt_Black_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_LimitedRun_TShirt_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_Red_R_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_Red_R_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_eS_Square_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_Red_R_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_Navy_R_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_Green_R_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_Charcoal_R_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_Brown_R_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_Bone_R_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_BlackWhite_R_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_BlackLeather_R_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_BlackLeather_R_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_BlackLeather_R_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_BlackGum_R_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_BlackGrey_R_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_eS_ShoesBox_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_Jenkem_TShirt_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_Default_Dirty", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_Heroin_TShirt_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Fallen_Atlas_B_sRGB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_Cargo_Pants_ColorIDMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_StinkerPudwill_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_TY_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_StinkerPudwill_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_UB_Hoodie_Down_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Hair_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_LB_Jeans_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Denim_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Denim_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Denim_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Denim_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Denim_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Denim_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Denim_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Denim_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Denim_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_eSFootwear_AccelOG_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_FT_eSFootwear_AccelOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_eSFootwear_AccelOG_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_eSFootwear_AccelOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Male_MidTop_GEN_Mask_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}ClothingMask" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Denim_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Denim_ICON_8", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Denim_ICON_7", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Denim_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Denim_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Denim_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Denim_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Denim_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Denim_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Denim_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_Jeans_Mask_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Denim_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_HD_Beanie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_BEAN_Fallen_Insignia_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_BEAN_Fallen_Insignia_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Fallen_Graphic_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Fallen_Graphic_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_FT_Fallen_Patriot", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Patriot_Marks_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Patriot_Marks_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Patriot_StoneBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Patriot_Marks_Wine", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Patriot_Marks_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_Fallen_Patriot", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_TankTop_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AFXX_Gen_Tank_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Tank" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Patriot_Marks_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Fallen_Graphic_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Fallen_Graphic_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_BEAN_Fallen_Insignia_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_BEAN_Fallen_Insignia_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Patriot_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Patriot_BlueWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Patriot_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Patriot_FullyBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Patriot_Crimson", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Patriot_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Patriot_BlackWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Patriot_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_BEAN_Fallen_Insignia_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_BEAN_Fallen_Insignia_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_HD_BEANIE_Creature_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Head_Beanie_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Head_Beanie_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_HD_BEANIE_Creature_Camo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_HD_BEANIE_Creature_Camo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_HOOD_Zero_DTL_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Camo_Denim_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Camo_Denim_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Gen_Cargo_Camo_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Gen_Cargo_Camo_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Gen_Cargo_Camo_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Gen_Cargo_Camo_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Camo_Denim_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Camo_Denim_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Gen_Cargo_Camo_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Camo_Denim_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Customizable_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Hoodie_ID_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}CustomMasks{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_CustomHoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Camo_Denim_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Camo_Denim_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Gen_Cargo_Camo_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Camo_Denim_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Gen_Cargo_Camo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Session_CFTW", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Session{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Session_CFTW_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Gen_Cargo_Camo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_LB_Cargo_Pants_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_UB_Tshirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_HOOD_Zero_DTLSingleSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_HD_BEANIE_Creature_Camo_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Head_Beanie_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jart_Manny_Skull_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Head_Beanie_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Yellow_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Green_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jart_Manny_Skull_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Head_Beanie_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Head_Beanie_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Head_Beanie_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Head_Beanie_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Session_CIAH_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_CINH", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Session{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AFXX_Gen_Tank_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Tank" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_UB_TankTop_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_BodyMaks_Tank", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_TankTop_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Hair_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Creature_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassicWide_Default_46_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Faded_Friday", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Faded{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Faded_Firday_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Wide_46", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Wide_46mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_149_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_149_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_AccelOG_Shoes_02_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_AccelOG_Shoes_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_eSFootwear_AccelOG_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_AccelOG_Shoes_01_RG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Jeans_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Jeans_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_159_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_ConicWide_Default_46_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_BurmanDeadPresidents_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_ConicWide_46", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_ConicWide_46mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_159_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WT_TY_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WD_SmearWarp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WD_Default_Scratch_A_SignedReworked_Contrast", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_144_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_BalancedDaewon_BC_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_ConicWide48_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_ConicWide_Default_48_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_BalancedDaewon_BC_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_144_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Scratches_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_StinkerPudwill_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_MaxTaylor_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Deck_NormalDistortion", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_TShirt_Mask_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_UB_TShirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Denim_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Cargo_Denim_Gen_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Cargo_Denim_Gen_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Denim_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Cargo_Denim_Gen_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Cargo_Denim_Gen_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Cargo_Denim_Gen_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Denim_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_Vul_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Cargo_Denim_Gen_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Denim_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Denim_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Denim_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_TShirt_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_TShirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}TShirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_Bender_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_Bender", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Cargo_Denim_Gen_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Denim_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Cargo_Denim_Gen_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Cargo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Cargo_Denim_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Schlaudie_Spryte", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SchlaudieSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Schlaudie_Spryte_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Jeans_ColorIDMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_GEN_CustomUB_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Master" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TieDye_Pattern_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_TankTop_WindMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_TankTop_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_TankTop_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_TankTop_B_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Camo_Pattern_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Fallen_Atlas_B_Embroidery_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_HD_BEANIE_Creature_G_Normal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_HD_BEANIE_Creature_G_Mixed_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Session_CFTW_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Session{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Session_CFTW_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Session{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_DG_CFTW", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Session{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_TShirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}TShirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jart_Manny_Skull_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_CIAH_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Session{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_CIAH_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Session{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_DG_CINH", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Session{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Hair", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Faded_Friday_8500_BC_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Faded{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_DG_Faded_Friday", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Faded{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_46mm_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_HG_Default_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_AccelOG_Shoes_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_AccelOG_Shoes_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_DeadPresidents_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_BalancedDaewon_BC_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_Jenkem_Shirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Reworked" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_BC_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Reworked" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_RogerSkateCo_Bender", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_sRGB_SchlaudieSkateCo_Padded", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}SchlaudieSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_BC_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Reworked" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Default_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Default_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Default_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_D1Raw_149_Mid_A_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Green_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Antilanta_Glow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Antilanta_Matrix_Glow_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Truck_BasePlate_Destructo_07", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_BasePlate_Destructo_Mid", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Truck_Hanger_Destructo_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Fallen_Insigina_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Fallen_Insigina_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Fallen_Insigina_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Sugar_Collegiate_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Sugar_Collegiate_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_SugarSkateCo_Collegiate_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SugarSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_SugarSkateCo_Collegiate_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SugarSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Fallen_Insigina_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Fallen_Insigina_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Fallen_Insigina_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Fallen_Insigina_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Fallen_Insigina_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Destructo_D1_149_Mid", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Male_Hoodie_Sweather_Mask_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}ClothingMask" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Jacket_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Jacket" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_Fallen_Trooper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Trooper_ChrisCole_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Trooper_ChrisCole_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Trooper_ChrisCole_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Trooper_ChrisCole_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Trooper_ChrisCole_White", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Trooper_ChrisCole_CamelBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Trooper_ChrisCole_BlackWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Trooper_ChrisCole_BlackRed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Trooper_ChrisCole_BlackPurple", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_FT_Fallen_Trooper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Trooper_ChrisCole_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Jacket_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_UB_WindBreaker_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WD_Default_Scratch_A_SignedReworked", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Antilanta_Glow_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "New_Graph_TEX_DG_Anti_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_BasePlate_Destructo_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_BasePlate_Destructo_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_BasePlate_Destructo_ID", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_BasePlate_Destructo_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Truck_BasePlate_Destructo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Truck_BasePlate_Destructo_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_Destructo_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_Destructo_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_Destructo_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Truck_Hanger_Destructo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_sRGB_SugarSkateCo_Padded", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}SugarSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Truck_Hanger_Destructo_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Jacket_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Reworked" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Jacket_WindMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Jacket_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Jacket_02_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_FEET_Fallen_Atlas_B_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Jacket_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Jacket" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_LongShirt_Mask_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_UB_Shirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AFXX_Gen_LongShirt_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}LongShirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_Shirt_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Hoodie_00_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_NoComply_CargoPants_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_NoComply_Cargo_Pants_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Hoodie_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shirt_WindMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shirt_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shirt_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shirt_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_NoComply_CargoPants_02_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AFXX_Gen_LongShirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}LongShirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Destructo_Circle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Destructo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_Destructo_Circle_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_HOOD_Zero_EyeBall_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_HOOD_Zero_Eyeball", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Bomber_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Hoodie_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_Fallen_Bomber", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Bomber_Olive", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Bomber_White", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Bomber_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Bomber_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Bomber_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_FT_Fallen_Bomber", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Hoodie_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_BEAN_Destructo_Trifecta", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Destructo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_BEAN_Destructo_Trifecta_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_Destructo_sRGB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Destructo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Bomber_RWTF_Purple", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_sRGB_ZERO_Padded", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_C_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_Destructo_Stencil", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Destructo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Bomber_RWTF_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassic_Default_36_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassic_Default_32_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_SatoriWheels_Classic_52_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_SatoriWheels_Conical_54_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_SatoriWheels_Classic_51_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_BoardyCakes_49_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_BoardyCakes_48_5_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_BoardyCakes_44_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_BoardyCakes_39_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_129_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_GrindKing_Spark_149", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_GrindKing_Kre-per_149", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_GrindKing_Kre-per_144", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Hanger_GrindKing_GK6_144", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_BasePlate_GrindKing_Kre-Per", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_BasePlate_GrindKing_GK6", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Destructo_D1_144_Mid", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Destructo_D1_139_Mid", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Destructo_D1_139_Low", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_BasePlate_Destructo_Low", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_eSFootwear_AccelOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Staticmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Roger_SausageParty_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_SamuraiPunk_TShirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_NoComply_TShirt_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_SamuraiPunk_TShirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_NoComply_TShirt_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_NoComply_TShirt_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ESP_TShirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ESP_TShirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ESP_TShirt_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ESP_TShirt_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ESP_TShirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ESP_TShirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ShirtJenkem_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ESP_TShirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Satori_LilNuggs_GooBall_54_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_BR_Default_A_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bearings{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_BoardyCakes_PrincessCut_49", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_BoardyCakes_FatBundts_44", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WheelTrans_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Default_EMPTY", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Truck_GrindKing_Kre-per_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Truck_GrindKing_GK6_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Truck_GrindKing_Kre-per_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Truck_GrindKing_GK6", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_HeroinSkateboard_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SatoriWheels_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_WheelTranslucent_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "New_Graph_outputwheelwarptest", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_WheelTrans_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_BasePlate_GrindKing_Kre-per_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_BasePlate_GrindKing_Kre-per_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_BasePlate_GrindKing_Kre-per_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_HeroinSkateboard_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_GrindKing_GK6_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_GrindKing_GK6_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_GrindKing_GK6_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_GrindKing_GK6_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_GrindKing_GK6_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_GrindKing_GK6_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_GrindKing_Kre-per_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_GrindKing_Kre-per_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_GrindKing_Kre-per_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Default_EMPTY_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_ComplyShirt_BrightBlue_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_LimitedRun_TShirt_03_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_LimitedRun_TShirt_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_FlatFace_GreenTShirt_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_FlatFace_TanTShirt_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_FlatFace_PurpleTShirt_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ESP_TShirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_Jenkem_TShirtBlack_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_FullBody_Male02_Base_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_EyeRefractive_Male02_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Bomber_RWTF_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Bomber_RWTF_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_Bomber_RWTF_Grey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Bomber_RWTF_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Satori_Calssic_Elephant_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Satori" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Denim_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Satori_Classic_Elephant_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Satori_Classic_Elephant_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_TheEasy_Purple", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_TheEasy_ChrimsonWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_TheEasy_BlueGum", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_TheEasy_BlackWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_Shoe_Fallen_TheEasy_BlackGum", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Roger_PartyYourBunsOff", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Roger_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_UB_Sweather_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Tank_Creature_Blank", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_Stencil_RogerSkateCo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Hat_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_Shirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFFXX_Beanie_Base_Inst", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Hat_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFFXX_Beanie_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DL_DeckLayers_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DL_Default_07_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_ACCES_AFXX_Glasses_Aviator", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}BaseMesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_ACCES_AFXX_Glasses_PitVisor", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}BaseMesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_ACCES_AFXX_Glasses_Round", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}BaseMesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_ACCES_AFXX_Glasses_Square", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}BaseMesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_ACCES_AFXX_Glasses_StrayBand", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}BaseMesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_ACCES_AMXX_Glasses_Aviator", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}BaseMesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_ACCES_AMXX_Glasses_PitVisor", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}BaseMesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_ACCES_AMXX_Glasses_Round", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}BaseMesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_ACCES_AMXX_Glasses_Square", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}BaseMesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_ACCES_AMXX_Glasses_StrayBand", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}BaseMesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_ACCES_EYES_Glasses_Aviator", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_ACCES_EYES_Glasses_PitVisor", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_ACCES_EYES_Glasses_Round", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_ACCES_EYES_Glasses_Square", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_ACCES_EYES_Glasses_StrayBand", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_ACCES_EYES_Glasses_Frame_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_ACCES_EYES_Glasses_Frame_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_ACCES_EYES_Glasses_Frame_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_ACCES_EYES_Glasses_Frame_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_ACCES_EYES_Glasses_Frame_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_ACCES_EYES_Glasses_Frame_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_ACCES_EYES_Glasses_Frame_F1", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_ACCES_EYES_Glasses_Glas_Shaded", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_ACCES_EYES_Glasses_Glas_Shaded_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_ACCES_EYES_Glasses_Glas_Shaded_iridescent", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_ACCES_EYES_Glasses_Glas_Shaded_iridescent_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_ACCES_EYES_Glasses_Frame_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Master" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_ACCES_EYES_Glasses_Glas_Optical_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Master" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_ACCES_EYES_Glasses_Glas_Shaded_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Master" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Glasses_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Accessories{Path.DirectorySeparatorChar}Eyes{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Adidas_Logo_Inverted", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Adidas{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_Adidas_3MCVulc_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Adidas{Path.DirectorySeparatorChar}Staticmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Almost", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_BlackGold", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}BlackGold{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Blind", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Blind" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BoardyCakes_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_CityStars", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_DC", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}DC{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Deviant", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Deviant{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Diz", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_BRN_eS_Square3_ShoeProp_L", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Staticmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_eS_RedSticker_Decal", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Stickers{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_eS_Footwear_RED_Logo_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Stickers{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_eS_Footwear_RED_Logo_RG", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Stickers{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_FTP", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_Globo_Mahalo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Globe{Path.DirectorySeparatorChar}Staticmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Grizzly", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_01_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_01_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_02_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_02_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_03_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_03_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_01_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_01_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_02_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_02_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_03_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_03_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_01_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_01_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_02_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_02_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_03_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_03_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}HeroinSkateboards{Path.DirectorySeparatorChar}NonCustomizationAssets{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Jacuzzi", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Logo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_LimitedRun_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_LiquidDeath", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}LiquidDeath{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Monarch", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_NBS", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_PlanB", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_RumCom_BlackHoodieLarge_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}RumCom{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_SandlotTimes", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Spitfire", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_SponsorMe", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Tensor", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Thunder", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_TMNT", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BrandLogo_Venture", RelativePath = $"Customization{Path.DirectorySeparatorChar}BrandAssets{Path.DirectorySeparatorChar}Venture{Path.DirectorySeparatorChar}Logos" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Body", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Body_Female02_MT", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Body_Female02_RG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Female02_Hairs_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Female02_Hairs_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Body_Underwear_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Cap_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Cap_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Cap_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Eye_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Hair_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Hair_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Body_Mask_Arms", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}ClothingMask" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Body_Mask_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}ClothingMask" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shoes_Vul_A_eS_Square3_Burgundy_R_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eS{Path.DirectorySeparatorChar}eS_Square3{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_eSFootwear_Explorer", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_eSFootwear_Sal23", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_Shoes_Vul_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Shoes_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Generic{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Shoes_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Generic{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Shoes_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Generic{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_Shoes_Cup_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Generic{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_Shoes_Vul_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Generic{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Shoes_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Generic{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Shoes_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Generic{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Shoes_ID", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Generic{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Shoes_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Generic{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LimitedRun_Cap_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LimitedRun_Cap_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_HD_Hair_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hair_C" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_HD_Hair_B_Scalp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hair_C" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_HD_Hair_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hair_C" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX02_Hair_Dreads_TMP", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Hat_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX02_Hair_TMP_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_HairDreads__TMP", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Hat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Hat_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Hat_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Cap_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Cap_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Cap_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Cap_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Cap_D_PhysicsAsset", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Cap_NoHair_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Hair_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Hair_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Hat_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Hat_NoHair_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_Hair01_Female02_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_GEN_HD_Hair_C_ALPHA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Hair_C" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_GEN_HD_Hair_C_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Hair_C" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_GEN_HD_Hair_C_Depth", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Hair_C" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_GEN_HD_Hair_C_ID", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Hair_C" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_GEN_HD_Hair_C_Root", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Hair_C" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CaucasianFemale_Hair_ALPHA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Hair_D" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CaucasianFemale_Hair_BaseColor", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Hair_D" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CaucasianFemale_Hair_Depth", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Hair_D" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CaucasianFemale_Hair_ID", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Hair_D" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CaucasianFemale_Hair_Root", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Hair_D" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Cap_02_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Cap_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Cap_02_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Cap_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Cap_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Cap_ID", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Cap_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Hat_02_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Hat_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Hat_02_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Hat_02_RG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_Socks_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_Socks_A_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_Socks_A_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_Socks_A_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_Socks_A_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_Socks_A_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_Socks_A_07", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_Socks_A_08", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_Socks_A_09", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_Socks_A_10", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_GEN_LB_Shorts_Socks_A_11", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Socks_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Socks_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Socks_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_Socks_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Short", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Short_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Short_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Short_White", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX_Socks", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_LB_Shorts_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_LB_Shorts_Socks_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_baggypants_mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Jeans_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Jeans_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_LB_Shorts_Female_BodyMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Pants_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Pants_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Pants_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Pants_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Short_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Short_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Short_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Short_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Short_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Socks_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Socks_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_LB_AFXX_BaggyPants_ID_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_LB_AFXX_BaggyPants_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_LB_AFXX_BaggyPants_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_Hoodie_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_Hoodie_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_Hoodie_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_Hoodie_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_Hoodie_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_TShirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_TShirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_TShirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_TShirt_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_TShirt_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_TShirt_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_ESP_TShirt_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LimitedRun_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LimitedRun_Hoodie_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LimitedRun_Hoodie_c", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LimitedRun_TShirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LimitedRun_TShirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AFXX_LimitedRun_TShirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AFXX_Gen_LongShirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}LongShirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AFXX_Gen_LongShirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}LongShirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AFXX_Gen_Tank_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Tank" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AFXX_Gen_Tank_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Tank" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AFXX_Gen_Tank_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Tank" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AFXX_Gen_Tank_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Tank" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_Tank_Mask_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_BodyMaks_ShortsSocks", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Gen_TShirt_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_LS_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shirt_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Shirt_ID", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_TankTop_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_TankTop_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_TankTop_RG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_UB_GEN_LSShirt_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_UB_GEN_LSShirt_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Hair", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Hair_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AFXX02_Hair_TMP_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX02_Hair_TMP", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Dreadlocks", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Hairs02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Hairs02_Cap", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_02_Hairs_Alpha", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_02_Hairs_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_02_Hairs_BaseColor", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_02_Hairs_Normal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_02_Hairs_Roughness", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Dreads_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Dreads_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Hair_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Hair_GL", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Hair_GR", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Hair_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Hair_SP", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Male03_Body_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Male03" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Hair_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Body_Base{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Mask_BaggyPants", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}BodyMasks" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PumpkinMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}BodyMasks" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_GEN_TShirtBaggy_BodyMask_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}BodyMasks" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Mask_Tank_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}BodyMasks" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_LB_AMXX_Shorts_BodyMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}BodyMasks" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Creature_Shoe_Prison", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_eSFootwear_AccelOG_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_eSFootwear_AccelOG_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_eSFootwear_AccelOG_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_eSFootwear_AccelOG_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_eSFootwear_Explorer_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_eSFootwear_Sal23_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_eSFootwear_Sal23_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_eSFootwear_Sal23_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_eSFootwear_Sal23_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_FT_eSFootwear_Explorer", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_FT_eSFootwear_Sal23", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_AccelOG_Shoes_03_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_AccelOG_Shoes_03_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_AccelOG_Shoes_04_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_AccelOG_Shoes_05_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_AccelOG_Shoes_06_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Explorer_Shoes_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Explorer_Shoes_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Explorer_Shoes_01_RG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Sal23_Shoes_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Sal23_Shoes_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Sal23_Shoes_01_RN", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Sal23_Shoes_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Sal23_Shoes_03_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Sal23_Shoes_04_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}eSFootwear{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_Globe_Mahalo_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Globe{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_FT_Globe_Mahalo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Globe{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Globe_Mahalo_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Globe{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Globe_Mahalo_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Globe{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Globe_Mahalo_01_RG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Globe{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_Vulc_Shoes_2_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_Vulc_Shoes_2_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_VulcShoes_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_VulcShoes_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_GEN_VulcShoes_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_FT_Shoes_Vul_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Shoes_ID_v2", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_Jenkem_Cap_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_Jenkem_Cap_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_Jenkem_Cap_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_HD_Cap_Jenkem_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_HD_Cap_Jenkem_No_Hair_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_JenkemCap_Black_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_JenkemCap_Green_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_JenKemCap_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_JenkemCap_Yellow_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LimitedRun_Cap_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LimitedRun_Cap_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_LimitedRun_Cap_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_LimitedRunCap_Black_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_LimitedRunCap_Black_RG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_LimitedRunCap_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_GEN_HD_Pumpkin", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_Hairs", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_HD_Beanie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_HD_Cap_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_HD_Hair_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PumpkinMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PumpkinMask_Anim", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PumpkinMask_PhysicsAsset", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Skelmeshes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_NoComply_CargoPants", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_NoComply_CargoPants_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Cargo_Pants_02_D_windMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Cargo_Pants_02_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Cargo_Pants_03_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Cargo_Pants_03_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Cargo_Pants_04_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_StraightCut_02_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_StraightCut_02_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_StraightCut_03_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_StraightCut_03_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_StraightCut_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_StraightCut_Gloss", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_StraightCut_RG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_LB_AMXX_BaggyPants_ID_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_LB_AMXX_BaggyPants_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_LB_AMXX_BaggyPants_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_Hoodie_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_Hoodie_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_Hoodie_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_Hoodie_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_Hoodie_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_TShirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_TShirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_TShirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_TShirt_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_TShirt_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_TShirt_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_ESP_TShirt_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Hoodie_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Hoodie_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Hoodie_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Hoodie_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Hoodie_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ESP_TShirt_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ESP_TShirt_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Shirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Shirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Shirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Shirt_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Shirt_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Shirt_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_ESP_Shirt_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_FlatFace_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_FlatFace_Hoodie_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_AMXX_ESP_TShirt_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_FlatFace_Shirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_FlatFace_Shirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_FlatFace_Shirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_FlatFace_Shirt_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_FlatFace_MikenMortyTShirt_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_FlatFace_PurpleHoodie_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_FlatFace_TanHoodie_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_Heroin_Shirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_Heroin_Shirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_Jenkem_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_Jenkem_Shirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_JenkemHoodie_Black_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Jenkem{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LimitedRun_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LimitedRun_Hoodie_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LimitedRun_Hoodie_c", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LimitedRun_TShirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LimitedRun_TShirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_LimitedRun_TShirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_LimitedRun_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_LimitedRun_Hoodie_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_LimitedRun_Hoodie_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_LimitedRun_Shirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_LimitedRun_Shirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_LimitedRun_Shirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_LimitedRun_Hoodie_01_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_LimitedRun_Hoodie_02_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_LimitedRun_Hoodie_03_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_NoComply_TShirt_10", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_NoComply_Shirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_NoComply_Shirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_NoComply_Shirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_NoComply_Shirt_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_NoComply_Shirt_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_NoComply_Shirt_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_NoComply_Shirt_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_NoComply_Shirt_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_NoComply_Shirt_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_NoComply_Shirt_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Material{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_ComplyShirt_Yellow_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}No-Comply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_SamuraiPunk_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_SamuraiPunk_Hoodie_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_SamuraiPunk_TShirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_SamuraiPunk_TShirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_SamuraiPunk_TShirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_SamuraiPunk_TShirt_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_SamuraiPunk_TShirt_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_SamuraiPunk_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_SamuraiPunk_Hoodie_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_SamuraiPunk_Shirt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_SamuraiPunk_Shirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_SamuraiPunk_Shirt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_SamuraiPunk_Shirt_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_SamuraiPunk_Shirt_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SamuraiPunk{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Hoodie_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Hoodie_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Hoodie_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Hoodie_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Hoodie_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Hoodie_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Jacket_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Jacket" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_Jacket_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Jacket" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Cloth_AMXX_GEN_TShirt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}TShirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_B_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_B_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_C_ID", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_C_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Hoodie_Gloss", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Jacket_AO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Jacket_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Jacket_ID", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Session_Hoodie_Heather_Grey_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_BaseMesh_BoxArtShirt_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Reworked" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_BC_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Reworked" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_BaseMesh_Shirt_Specular", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_BC_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_BC_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_EffectMG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Gen_TShirt_RMO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Costume_LiquidDeath", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Costume_TMNT_Donatello", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Costume_TMNT_Leo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Costume_TMNT_Mikey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Costume_TMNT_Raph", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_Adidas_MatchBreakSuper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_Adidas_NorasShoe_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_Adidas_NorasShoe_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_Adidas_ProModel", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_DC_KalisVulc_BKW", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_DC_KalisVulc_XSSH", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_DC_OGLynx_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_DC_OGLynx_GO6", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_Fallen_Ripper_Cole_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_FT_Fallen_Ripper_Cole_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_Head_Creature_PandaHead", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_Head_BucketHat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_Head_BeanieHat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_Head_Cap_DGK_FiveStar", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}DGK" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_Head_Cap_DGK_Unseen", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}DGK" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_Head_Cap_Diz_CircleSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_Head_Cap_Diz_TecSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_Head_Cap_Flip_BubbleLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Flip" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_Head_Cap_Flip_FlipScript", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Flip" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_HD_CAP_FTP_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}FTP" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_HD_BEANIE_LiquidDeath_ThirstHunterSkullWarmer", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_HD_CAP_NBS_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_HD_BEANIE_PlanB_Tab", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_HD_CAP_PlanB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_HD_CAP_Suck_Ratcoste", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_LB_Shorts_Denim", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Almost_DoubleBar", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Almost_DoubleBar", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Almost_Intertwine", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Antilanta_Matrix", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Antilanta" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_BlackGold_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_BlackGold_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Blind_OgRippedSsPricePoint", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Blind" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Blind_VintageBananaReaper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Blind" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Brandalised_GirlWithStick", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HooideBaggy_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_LS", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_ShirtBaggy_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_DC_BigLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_DC_PocketT", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_LS_DC_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_DGK_Bomb", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DGK" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_DGK_FiveStar", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DGK" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_DGK_Guadalupe", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DGK" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_DGK_LoSide", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DGK" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_Hood_Diz_Ind_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_Hood_Diz_Ind_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_Hood_Diz_TecSIgn", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_LS_Diz_TechSign", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Diz_Army", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Diz_Ind_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Diz_Ind_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Diz_KeepItEvil", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Diz_NoSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Diz_Script", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Diz_SparyWings_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Diz_SparyWings_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Diz_TecSign", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Flip_FlipBubble", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Flip" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Flip_FlipScript", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Flip" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Flip_LoveShroom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Flip" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Flip_OgShroom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Flip" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Flip_FlipBubble", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Flip" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Flip_FlipScript", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Flip" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Flip_LoveShroom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Flip" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Flip_OgShroom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Flip" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_FTP_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}FTP" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_FTP_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}FTP" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_FTP_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}FTP" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Jacuzzi_Flavour", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Jacuzzi_Koi", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_AMXX_UB_TEE_Jacuzzi_ScaredWeed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_Krux_KruxRainbow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_Krux_KruxRainbow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_Hood_LiquidDeath_HydrateorDieRastaHoodie", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_LS_LiquidDeath_InstantDeathSweatshirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_LiquidDeath_ExclusiveDeathTee", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_LiquidDeath_HydrateorDieRedrumTee", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_Hood_Monarch_Atelier_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_Hood_Monarch_Atelier_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_Hood_Monarch_Atelier_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_LS_Monarch_Atelier_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_LS_Monarch_Atelier_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_LS_Monarch_Atelier_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TBaggy_Monarch_Horus_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TBaggy_Monarch_Horus_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TBaggy_Monarch_Horus_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TBaggy_Monarch_Horus_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_HOOD_NBS_NextLevel", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_HOOD_NBS_Ten", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_LS_NBS_WeStillHere", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_NBS_CurveLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_NBS_LifeBeginsHere", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_NBS_NeverGiveUp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_NBS_NextLevel", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_NBS_Support", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_NBS_Ten", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_NBS_Wings", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_HOOD_OJ_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}OJ" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_OJ_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}OJ" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_Hood_PlanB_FullLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_SWT_PlanB_LogoCrew", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_PlanB_FullLogoTee", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_PlanB_LogoLeft", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_PlanB_LogoTee", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_SM_Colincoln", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SponsorMe" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_SM_LesToasters", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SponsorMe" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_TEE_SM_Mesa", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SponsorMe" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_Hood_Suck_Ratcoste", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_Hood_Suck_SuckItHard", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Suck_AcidRat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Suck_AfterSex", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Suck_Botanical", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Suck_HomeSweetHome", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Suck_LadySmoker", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Suck_LeRat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Suck_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Suck_Spoon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_Suck_SuckIt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_LS_TMNT_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_LS_TMNT_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_LS_TMNT_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_LS_TMNT_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_LS_TMNT_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_TMNT_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_TMNT_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_TMNT_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_TMNT_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_TMNT_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_TMNT_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_TMNT_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_UB_TEE_TMNT_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Camo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Camo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Camo_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Camo_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Camo_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Camo_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Camo_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Camo_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Cotton_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Cotton_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Cotton_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Cotton_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Cotton_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Cotton_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Cotton_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Cotton_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Cotton_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Cotton_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Denim_Gen_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Denim_Gen_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Denim_Gen_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Denim_Gen_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Denim_Gen_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Denim_Gen_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Denim_Gen_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Denim_Gen_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Denim_Gen_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Baggy_Denim_Gen_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Cotton_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Cotton_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Cotton_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Cotton_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Cotton_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Cotton_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Cotton_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Cotton_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Cotton_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Cotton_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Camo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Camo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Camo_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Camo_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Camo_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Camo_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Camo_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Gen_Jeans_Camo_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Jeans" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Shorts_Denim_Gen_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shorts" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Pants_Prison", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Prison" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_HOOD_Almost_DoubleBarStandard_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_HOOD_Almost_DoubleBarStandard_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Almost_DoubleBarStandard_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Almost_DoubleBarStandard_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Almost_Intertwine_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Almost_Intertwine_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_BlackGold_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_BlackGold_BA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_BlackGold_BB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_BlackGold_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_BlackGold_CA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_BlackGold_CB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_BlackGold_CC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_LS_DC_Logo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_LS_DC_Logo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_GEN_HoodBaggy_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_GEN_LS_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_GEN_ShirtBaggy_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_LS_Diz_TechSign", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Diz_Army", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Diz_Ind_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Diz_Ind_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Diz_KeepItEvil", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Diz_NoSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Diz_Script", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Diz_SparyWings_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Diz_SparyWings_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Diz_TecSign", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_FTP_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}FTP" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_FTP_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}FTP" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_FTP_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}FTP" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Jacuzzi_Flavour_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Jacuzzi_Flavour_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Jacuzzi_Koi_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Jacuzzi_Koi_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Jacuzzi_ScaredWeed_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Jacuzzi_ScaredWeed_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_HOOD_Krux_Rainbow_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_LongSleeve_Krux_KruxRainbow_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_LongSleeve_Krux_KruxRainbow_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Krux_Rainbow_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Krux_Rainbow_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_LiquidDeath_InstantDeathSweatshirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AfXX_UB_TEE_LiquidDeath_ExclusiveDeathTee", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_LiquidDeath_HydrateorDieRedrumTee", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_Hood_Monarch_Atelier_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_Hood_Monarch_Atelier_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_Hood_Monarch_Atelier_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_Monarch_Atelier_LS_Tee_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_Monarch_Atelier_LS_Tee_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_Monarch_Atelier_LS_Tee_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TBaggy_Monarch_Horus_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TBaggy_Monarch_Horus_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TBaggy_Monarch_Horus_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TBaggy_Monarch_Horus_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_LS_NBS_WeStillHere_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_LS_NBS_WeStillHere_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_CurveLogo_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_CurveLogo_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_CurveLogo_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_CurveLogo_AD", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_CurveLogo_AE", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_CurveLogo_AF", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_LifeBeginsHere_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_LifeBeginsHere_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_LifeBeginsHere_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_LifeBeginsHere_AD", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_LifeBeginsHere_AE", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_LifeBeginsHere_AF", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_NeverGiveUp_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_NextLevel_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_NextLevel_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_NextLevel_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_Support_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_Support_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_Support_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_Support_AD", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_Support_AE", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_Support_AF", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_TenTee_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_TenTee_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_TenTee_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_TenTee_AD", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_TenTee_AE", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_TenTee_AF", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_Wings_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_NBS_Wings_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_HOOD_OJ_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}OJ" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_OJ_Logo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}OJ" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_OJ_Logo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}OJ" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_PlanB_AthleticTee_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_PlanB_AthleticTee_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_PlanB_AthleticTee_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_PlanB_FullLogoTee_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_PlanB_FullLogoTee_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_PlanB_FullLogoTee_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_PlanB_LogoTee_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_PlanB_LogoTee_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_PlanB_LogoTee_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_SM_Colincol", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SponsorMe" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_SM_LesToasters", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SponsorMe" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_SM_Mesa", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SponsorMe" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_AcidRat_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_AcidRat_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_AdultEnt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_AfterSex", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_Botanical", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_HomeSweetHome", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_LadySmoker", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_LeRat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_LogoTee_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_LogoTee_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_Spoon_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_Spoon_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_SuckIt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_SuckIt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_Suck_SuckIt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_HD_BEANIE_LiquidDeath_ThirstHunterSkullWarmer", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_HD_BEANIE_PlanB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Camo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Camo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Camo_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Camo_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Camo_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Camo_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Camo_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Camo_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Cotton_Gen_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Cotton_Gen_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Cotton_Gen_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Cotton_Gen_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Cotton_Gen_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Cotton_Gen_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Cotton_Gen_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Cotton_Gen_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Cotton_Gen_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Cotton_Gen_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Cotton_Gen_K", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Denim_Gen_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Denim_Gen_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Denim_Gen_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Denim_Gen_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Denim_Gen_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Denim_Gen_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Denim_Gen_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Baggy_Denim_Gen_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baggy" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Shorts_Denim_Gen_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shorts" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Denim_Gen_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Denim_Gen_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Denim_Gen_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Denim_Gen_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}StraightCut" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_STCU_Prison", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody{Path.DirectorySeparatorChar}Prison" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_HOOD_Almost_DoubleBarStandard_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_HOOD_Almost_DoubleBarStandard_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Almost_DoubleBarStandard_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Almost_DoubleBarStandard_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Almost_Intertwine_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Almost_Intertwine_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Antilanta_Matrix", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Antilanta" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_BlackGold_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_BlackGold_BA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_BlackGold_BB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_BlackGold_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_BlackGold_CA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_BlackGold_CB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_BlackGold_CC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Blind_OgRippedSsPricePoint_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Blind" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Blind_OgRippedSsPricePoint_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Blind" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Blind_OgRippedSsPricePoint_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Blind" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Blind_VintageBananaReaper_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Blind" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Blind_VintageBananaReaper_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Blind" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Blind_VintageBananaReaper_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Blind" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Creature_Blank", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_LS_DC_Logo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_LS_DC_Logo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_DC_BigLogo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_DC_BigLogo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_DC_PocketT_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_DC_PocketT_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_GEN_HoodBaggy_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_GEN_LS_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_GEN_ShirtBaggy_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Hood_Diz_Ind_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Hood_Diz_Ind_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Hood_Diz_TechSign", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_LS_Diz_TechSign", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Diz_Army", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Diz_Ind_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Diz_Ind_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Diz_KeepItEvil", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Diz_NoSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Diz_Script", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Diz_SparyWings_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Diz_SparyWings_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Diz_TecSign", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_FTP_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}FTP" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_FTP_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}FTP" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_FTP_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}FTP" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Jacuzzi_Flavour_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Jacuzzi_Flavour_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Jacuzzi_Koi_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Jacuzzi_Koi_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Jacuzzi_ScaredWeed_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Jacuzzi_ScaredWeed_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_HOOD_Krux_Rainbow_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_LongSleeve_Krux_KruxRainbow_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_LongSleeve_Krux_KruxRainbow_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Krux_Rainbow_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Krux_Rainbow_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Hood_LiquidDeath_HydrateorDieRastaHoodie", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_LiquidDeath_ExclusiveDeathTee", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_LiquidDeath_HydrateorDieRedrumTee", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_LiquidDeath_InstantDeathSweatshirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Hood_Monarch_Atelier_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Hood_Monarch_Atelier_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Hood_Monarch_Atelier_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Monarch_Atelier_LS_Tee_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Monarch_Atelier_LS_Tee_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Monarch_Atelier_LS_Tee_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TBaggy_Monarch_Horus_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TBaggy_Monarch_Horus_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TBaggy_Monarch_Horus_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TBaggy_Monarch_Horus_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_CurveLogo_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_CurveLogo_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_CurveLogo_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_CurveLogo_AD", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_CurveLogo_AE", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_CurveLogo_AF", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_LifeBeginsHere_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_LifeBeginsHere_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_LifeBeginsHere_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_LifeBeginsHere_AD", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_LifeBeginsHere_AE", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_LifeBeginsHere_AF", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_NeverGiveUp_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_NextLevel_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_NextLevel_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_NextLevel_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_Support_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_Support_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_Support_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_Support_AD", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_Support_AE", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_Support_AF", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_TenTee_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_TenTee_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_TenTee_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_TenTee_AD", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_TenTee_AE", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_TenTee_AF", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_Wings_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_NBS_Wings_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_HOOD_OJ_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}OJ" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_OJ_Logo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}OJ" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_OJ_Logo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}OJ" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_PlanB_AthleticTee_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_PlanB_AthleticTee_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_PlanB_AthleticTee_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_PlanB_FullLogoTee_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_PlanB_FullLogoTee_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_PlanB_FullLogoTee_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_PlanB_LogoTee_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_PlanB_LogoTee_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_PlanB_LogoTee_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_SWT_PlanB_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_SWT_PlanB_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Roger_Doodler", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Roger_SausageParty_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Roger_SausageParty_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Roger_SausageParty_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}RogerSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Satori_Rising_Sun", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Satori" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Satori_Tea_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Satori" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Satori_Wheel_Movement", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Satori" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Schlaudie_LilLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SchlaudieSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Schlaudie_Pip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SchlaudieSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Schlaudie_PirateShip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SchlaudieSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_SM_Colincol", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SponsorMe" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_SM_LesToasters", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SponsorMe" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_SM_Mesa", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SponsorMe" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Hood_Suck_Ratcoste", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Hood_Suck_SuckItHard", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_AcidRat_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_AcidRat_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_AdultEnt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_AfterSex", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_Botanical", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_HomeSweetHome", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_LadySmoker", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_LeRat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_LogoTee_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_LogoTee_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_Spoon_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_Spoon_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_SuckIt_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_SuckIt_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Suck_SuckIt_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_SugarSkateCo_SugarGradient_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SugarSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_SugarSkateCo_SugarGradient_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SugarSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_SugarSkateCo_TieDye", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}SugarSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_TMNT_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_TMNT_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_TMNT_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_TMNT_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_TMNT_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_TMNT_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_TMNT_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_TEE_TMNT_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_LS_TMNT_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_LS_TMNT_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_LS_TMNT_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_LS_TMNT_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_LS_TMNT_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_TMNT_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_TMNT_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_TMNT_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_TMNT_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_TMNT_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_TMNT_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_TMNT_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_TMNT_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_HOOD_Zero_Ransom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_SWT_Zero_Army", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Zero_DamnItAll", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Zero_DamnItAll_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Zero_Ransom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Zero_Skull_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Zero_Skull_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Zero_StarDust", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Zero" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Costume_LiuqidDeath_LB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Costume_LiuqidDeath_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Costume_LiuqidDeath_UB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Costume_LD_Hands", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX__FEET_Adidas_Daweon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX__FEET_Adidas_HiPro", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX__FEET_Adidas_MatchBreakSuper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX__FEET_Adidas_Nora", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX__FEET_Adidas_Nora_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX__FEET_DC_OGLynx_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX__FEET_DC_OGLynx_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_FEET_DC_KalisVulc_BKW", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_FEET_DC_KalisVulc_XSSH", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_FEET_CupSoleShoes_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_FEET_CupSoleShoes_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_FEET_CupSoleShoes_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_FEET_CupSoleShoes_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_FEET_CupSoleShoes_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_FEET_CupSoleShoes_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_FEET_CupSoleShoes_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_GEN_FEET_CupSoleShoes_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX__FEET_Fallen_Ripper_ChrisCole_Red", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX__FEET_Fallen_Ripper_ChrisCole_White", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX__FEET_NewBalance", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}NewBalance" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_BucketHat_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}BucketHat" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_BucketHat_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}BucketHat" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_BucketHat_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}BucketHat" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_BucketHat_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}BucketHat" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_BucketHat_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}BucketHat" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_BucketHat_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}BucketHat" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_BucketHat_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}BucketHat" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_BucketHat_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}BucketHat" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_Beanie_Prison", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Prison" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Diz_CircleSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Diz_TecSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Fallen_Claw_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Fallen_Claw_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Fallen_HendrixPatch_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Fallen_HendrixPatch_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Fallen_Trademark_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Fallen_Trademark_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Fallen_Trademark_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Fallen_Trademark_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_FTP_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}FTP" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_ComingSoon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_NBS_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_PlanB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Suck_Ratcoste_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_HD_CAP_Suck_Ratcoste_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}Head{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_BlackGold_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_BlackGold_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Brandalised_Crime", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Brandalised_FollowYourDreams", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Brandalised_Guard", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Brandalised_What", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Brandalised_Bunny", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Brandalised_DreamBig", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Brandalised_GirlWithStick", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Brandalised_Rat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Brandalised" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Creautre_bracketA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Prison", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Prison" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Destructo_Circle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Destructo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Destructo_Iron", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Destructo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_TEE_Destructo_Star", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Destructo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_DLC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}DLC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_SWT_Faded_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Faded" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Faded_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Faded" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Fallen_Insignia_BlackEnzymatic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Fallen_Insignia_BlackWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Fallen_Insignia_PurpleBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Fallen_RiseWith_BlackWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Fallen_RiseWith_WhiteBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Fallen_Trademark2_GreyBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Fallen_Trademark2_SandBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Fallen_Trademark_BlackWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_Fallen_Trademark_GreenBeige", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_DeadHead_BlackWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_DeadHead_WhiteBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Desert_BlackWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Hendrix_BeigeBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Hendrix_BlackOrangeGradient", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Insignia_BurgundyWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Insignia_GeenBeige", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Insignia_WhiteViolet", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_NightFiend_BlackWashedWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Revolver_BlackGold", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Revolver_BlackWashedWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_RiseWith_BlackWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_RiseWith_PlumAqua", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Stripes_BlackGray", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Trademark_BlackOrange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Trademark_BlackPrinted", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Trademark_EnzymaticWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_Fallen_Trademark_PurpleLime", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_SWT_iDabble_80s_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}iDabble" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_SWT_iDabble_80s_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}iDabble" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_ComingSoon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_LS_ComingSoon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_ComingSoon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_NBS_NextLevel_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_NBS_NextLevel_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_NBS_Ten_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_NBS_Ten_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_UB_LS_NBS_WeStillHere_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_UB_LS_NBS_WeStillHere_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_PlanB_AA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_PlanB_AB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_PlanB_AC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_TY_CloudyHoodie", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_TY_HeadInTheClouds", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_HOOD_TY_SpotOn", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_TY_Center_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_TY_Center_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_TY_Center_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_TY_CloudIcon_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_TY_CloudIcon_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_TY_Cloudy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_TY_Rampage_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_TY_Rampage_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_TY_Stinker_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GEN_UB_TEE_TY_Stinker_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI{Path.DirectorySeparatorChar}GEN{Path.DirectorySeparatorChar}UpperBody{Path.DirectorySeparatorChar}ThankYou" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_BucketHat_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Master" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Panda_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Master" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_PandaFurCards_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Master" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Shoe_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Master" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_FEET_DC_LynxOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_FEET_Fallen_Ripper_ChrisCole", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_FT_Adidas_HiPro", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_FT_Adidas_MatchBreakSuper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_FT_Adidas_Nora", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_Fallen_TheEasy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_NewBalance", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_FT_Shoes_MidTop_Cup_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_FEET_DC_KallisVulc", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Beanie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_BeanieHat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_BucketHat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_HD_Panda", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_LB_BaggyPants_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_LB_ShortsCargo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}LowerBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_UB_HoodieBaggy_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_GEN_UB_LS", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AFXX_UB_TshirtBaggy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AFXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_FT_Adidas_NoraShoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_FEET_Fallen_Ripper_ChrisCole", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_FT_Adidas_HiPro", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_FT_Adidas_MatchBreakSuper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_FT_Fallen_TheEasy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_FT_NewBalance", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_FT_Shoes_MidTop_Cup_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_DC_OGLynxZero", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_FEET_DC_KallisVulc", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Feet{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_HD_BeanieHat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_HD_BucketHat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_HD_Panda", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}Head" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_LB_BaggyPants_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_LB_Shorts_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}LowerBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_UB_HoodieBaggy_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_UB_LS", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_UB_ShirtBaggy_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_GEN_UB_TankTop", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}AMXX{Path.DirectorySeparatorChar}UpperBody" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Costume_LiquidDeath", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Acessories2_Don", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Acessories_Don", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Eyes_Don", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Glasses_Don", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Shells_Don", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Skin_Don", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Teeth_Don", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Weapon_Don", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_01_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_02_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_02_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_03_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_04_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_04_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_05_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_05_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_06_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_07_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_07_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Donatello_Teeth_08_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Unique_TMNT_Donatello", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Unique_TMNT_Donatello_PhysicsAsset", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Donatello" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Acessories_Leo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_BeltsBands_Leo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Eyes_Leo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Shells_Leo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Skin_Leo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Teeth_Leo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Weapon_Leo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Leonardo_BC_Accessories_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Leonardo_BC_Bandage_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Leonardo_BC_Eyes_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Leonardo_BC_Shell_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Leonardo_BC_Teeth_07", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Leonardo_BC_Weapon_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Leonardo_NM_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Leonardo_NM_Accessories_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Leonardo_NM_Bandage_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Leonardo_NM_Shell_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Leonardo_NM_Weapon_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Unique_TMNT_Leonardo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Unique_TMNT_Leonardo_PhysicsAsset", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Leonardo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Acessories", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}MasterMats" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_BeltsBands_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}MasterMats" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Eyes_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}MasterMats" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Glasses", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}MasterMats" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Shells", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}MasterMats" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Mat_sketchLines", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}MasterMats" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Skin", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}MasterMats" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Teeth", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}MasterMats" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Weapon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}MasterMats" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Acessories_Mikey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Acessories_Waepon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_BeltsBands_Mikey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Eyes_Mikey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Shells_Mikey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Skin_Mikey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Teeth_Mikey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Michelangelo_Accessories_04_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Michelangelo_Accessories_04_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Michelangelo_Bandage_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Michelangelo_Bandage_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Michelangelo_Body_03_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Michelangelo_Body_03_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Michelangelo_Eyes_06_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Michelangelo_Shell_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Michelangelo_Shell_02_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Michelangelo_Teeth_07_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Michelangelo_Weapon_05_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Michelangelo_Weapon_05_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Unique_TMNT_Mikey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Unique_TMNT_Mikey_PhysicsAsset", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Mikey" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Acessories_Raph", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_BeltsBands_Raph", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Eyes_Raph", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Shell_Raph", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Skin_Raph", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Teeth_Raph", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Weapon_Raph", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_BurshStrokes_TMNT", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Raphael_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Raphael_02_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Raphael_03_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Raphael_04_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Raphael_05_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Raphael_Mask_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Raphael_Mask_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Raphael_Mask_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Raphael_Mask_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Raphael_Mask_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Raphael_Mask_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TX_Raphael_Mask_Teeth_07", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Unique_TMNT_Raph", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Unique_TMNT_Raph_PhysicsAsset", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}SkelMesh{Path.DirectorySeparatorChar}Costumes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Raph" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_Adidas_Daweon_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_Adidas_Daweon_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_Adidas_Daweon_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_Adidas_MatchBreakSuper_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_Adidas_MatchBreakSuper_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_Adidas_MatchBreakSuper_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_Adidas_Nora_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_Adidas_Nora_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_Adidas_Nora_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_Adidas_Nora_V2_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_AdidasHiPro_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_AdidasHiPro_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_AdidasHiPro_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Adidas" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Almost_sRGB_Atlas", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Almost" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_BlackGold_Atlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BlackGold" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Blind_sRGB_Atlas", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Blind" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Panda_BaseColor", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Panda" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Panda_BaseColor_Normal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Panda" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GEN_HD_Panda_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Panda" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_LB_Pants_StripeMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_GEN_HD_BEANIE_Creature_G_Base_color", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_GEN_HD_BEANIE_Creature_G_Normal_DirectX", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_LB_STCU_StripeMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_StripeMask_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Creature" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_DC_OGLynxZeroBGM_BodyMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Apparel_DC_Atlas", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DC_KalisVulc_BKW_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DC_KalisVulc_BKW_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DC_KalisVulc_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DC_KalisVulc_XSSH_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DC_KalisVulc_XSSH_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_FEET_DC_OGLynxZeroBGM_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_FEET_DC_OGLynxZeroBGM_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_FEET_DC_OGLynxZeroBGM_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_FEET_DC_OGLynxZeroGO6_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_DC_BigLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Tuque_AO_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Gen_Tuque_NM_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_H_GEN_BucketHat_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}BucketHat" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_H_GEN_BucketHat_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}BucketHat" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_GEN_UB_HoodieBaggy_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_GEN_UB_HoodieBaggy_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_GEN_UB_HoodieBaggy_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_GEN_UB_HoodieBaggy_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hoodie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_LS_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}LS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_GEN_LSShirt_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}LS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_GEN_LSShirt_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}LS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_UB_GEN_ShirtBaggy_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_UB_GEN_ShirtBaggy_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_UB_GEN_ShirtBaggy_Pocket_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_UB_TshirtBaggy_BodyMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_GEN_ShirtBaggy_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_GEN_ShirtBaggy_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_GEN_ShirtBaggy_Pocket_v2_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_GEN_ShirtBaggyPocket_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shirt" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CupSole_Shoes_ID_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shoes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CupSole_Shoes_ID_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shoes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_LB_Shorts_Female_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shorts" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_LB_Shorts_Female_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shorts" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_LB_AMXX_Shorts_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shorts" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_LB_AMXX_Shorts_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Shorts" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_UB_TEE_BAGGY_Stripe_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AtlasComingSoon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Cotton_Macro", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GEN_HD_CAP_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Tee_Pattern_StripesSmall", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TieDye_Pattern_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Default" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Diz_Atlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Diz_Atlas_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Diz" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DLC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}DLC" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Fallen_Ripper_Red_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Fallen_Ripper_Red_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Fallen_Ripper_Red_N", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Fallen_Ripper_White_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Fallen_Ripper_White_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Fallen_Ripper_White_N", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Fallen_Atlas_A_sRGB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Fallen" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Apparel_Atlas_FTP", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}FTP" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_iDabble_80s", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}iDabble" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Jacuzzi_ApparelAtlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Jacuzzi" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_sRGB_Krux_Padded", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Krux" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX__UnsexyThirstMurderer_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}LiquidDeath{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX__UnsexyThirstMurderer_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}LiquidDeath{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX__UnsexyThirstMurderer_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}LiquidDeath{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX__UnsexyThirstMurderer_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}LiquidDeath{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX__UnsexyThirstMurderer_02_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}LiquidDeath{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX__UnsexyThirstMurderer_02_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}LiquidDeath{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX__UnsexyThirstMurderer_Shoes_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}LiquidDeath{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX__UnsexyThirstMurderer_Shoes_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}LiquidDeath{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX__UnsexyThirstMurderer_Shoes_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}LiquidDeath{Path.DirectorySeparatorChar}Costume" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_LiquidDeath_Atlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}LiquidDeath" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Monarch_Atlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Monarch" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_NBS_ApparelAtlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}NBS" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AFXX_Feet_NewBalance_BodyMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}NewBalance" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_AMXX_Feet_NewBalance_BodyMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}NewBalance" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_NewBalance_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}NewBalance" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_NewBalance_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}NewBalance" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Feet_NewBalance_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}NewBalance" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_OJ_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}OJ" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_PlanB_ApparelAtlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_PlanB_ApparelAtlas_A_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}PlanB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_Stencil_SchlaudieSkateCo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}SchlaudieSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_SM_Atlas", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}SponsorMe" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Suck_ApparelAtlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Suck_ApparelAtlas_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Suck" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_Stencil_SugarSkateCO", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}SugarSkateCo" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_sRGB_ThankYou_Padded", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}ThankYouSkateboards" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_Stencil_ThankYou", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}ThankYouSkateboards" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_Testudinidae_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_ApparelAtlas_Testudinidae_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Apparel{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}TMNT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Character_LODSettings", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Unique_Annie_Shorts", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Annie" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Appleyard_Hoodie", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Appleyard{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Appleyard_Hoodie", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Appleyard{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_UB_Globe_StencilAtlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Appleyard{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "AMXX_Unique_ChrisCole_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}ChrisCole" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_ChrisCole_Hoodie", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}ChrisCole" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_ChrisCole_Pants", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}ChrisCole" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_ChrisCole_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}ChrisCole" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Unique_ChrisCole_Hoodie_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}ChrisCole" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Unique_ChrisCole_Hoodie_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}ChrisCole" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Donovan", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Donovan" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Donovan", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Donovan" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Uniqe_JahmirBrown_Deck", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}JahmirBrown" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_JahmirBrown_Hoodie", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}JahmirBrown" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Unique_Jahmir_Hoodie", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}JahmirBrown" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Unique_JahmirHoodieArt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}JahmirBrown" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_LouieBarletta_Shirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}LouieBarletta" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_LouieBarletta_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}LouieBarletta" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MIA_Unqiue_Louie_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}LouieBarletta" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CONVERSE_AllStarPro_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}LouieBarletta" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CONVERSE_AllStarPro_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}LouieBarletta" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Converse_OneStarPro_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}LouieBarletta" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Unqiue_Louie_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}LouieBarletta" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Unqiue_Louie_Swater", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}LouieBarletta" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Uniqe_Nora_Pants", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Nora" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_Nora_Shirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Nora" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_Nora_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Nora" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Unique_NoraVasconcellos_Pants", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Nora" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Unique_NoraVasconcellos_Shirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Nora" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Unique_Nora_Shirt_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Nora" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Unique_Nora_Shirt_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Nora" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Unique_NoraVasconcellos_Pants", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Nora" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Unique_NoraVasconcellos_Shirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Nora" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_LB_PED_PantsBaggy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_LB_PED_PantsTight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_PED_HoodieBaggy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_PED_HoodieTight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_PED_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_PED_TeeBaggy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GEN_UB_PED_TeeTight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CA_Pedestrains", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Curves" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CV_Pedestrains_LB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Curves" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CV_Pedestrains_UB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Curves" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Ped_PantsBaggy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}LB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_LB_Ped_PantsTight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}LB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Ped_PantsBaggy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}LB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_LB_Ped_PantsTight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}LB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Pedestrain_LowerBody_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Pedestrain_Shoes_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Pedestrain_UpperBody_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_FEET_Ped_Shoe1", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shoes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_FEET_Ped_Shoe", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}Shoes" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_Ped_HoodBaggy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}UB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AFXX_UB_Ped_TeeBaggy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}UB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Ped_HoodBaggy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}UB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Ped_HoodTight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}UB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Ped_TeeBaggy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}UB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_AMXX_UB_Ped_TeeTight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}Pedestrians{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}UB" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_RyanThompson_Pants", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}RyanThompson" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_RyanThompson_Shirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}RyanThompson" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Unique_RyanThompson_Shirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}RyanThompson" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_Samarria_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SamarriaBrevard" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SessionOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_HG_SessionPrison_Scratched_01_Inst", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_GT_SessionOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "SKXX_SkateMesh_Deck", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "SKXX_SkateMesh_Truck_Base", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "SKXX_SkateMesh_Truck_Hanger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "SKXX_SkateMesh_Wheel", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_SessionPrison_Scratched_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_SessionPrison_Scratched_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_SessionPrison_Scratched_01_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}SessionOG{Path.DirectorySeparatorChar}Skateboard{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_Torey_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}ToreyPudwill" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_Unique_ToreyPudwill_Shirt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}ToreyPudwill" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Unique_Torrey_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}ToreyPudwill" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Unique_Torrey_Shoes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Characters{Path.DirectorySeparatorChar}Uniques{Path.DirectorySeparatorChar}ToreyPudwill" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Aviator_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Aviator_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Aviator_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Aviator_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Aviator_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Aviator_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Aviator_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_PitVisor_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_PitVisor_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_PitVisor_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_PitVisor_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_PitVisor_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_PitVisor_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_PitVisor_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Round_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Round_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Round_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Round_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Round_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Round_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Round_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Square_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Square_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Square_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Square_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Square_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Square_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_Square_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_StrayBand_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_StrayBand_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_StrayBand_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_StrayBand_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_StrayBand_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_StrayBand_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_ACCES_EYES_Glasses_StrayBand_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_Hoodie_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_Hoodie_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_Hoodie_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_Hoodie_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_Hoodie_E_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_Hoodie_F_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_TShirt_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_TShirt_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_TShirt_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_TShirt_E_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_TShirt_F_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ESP_TShirt_G_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_eSFootwear_AccelOG_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_eSFootwear_AccelOG_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_eSFootwear_AccelOG_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_eSFootwear_Sal23_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Cup_A1_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Cup_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Mid_Top_Cup_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Mid_Top_Cup_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Mid_Top_Cup_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Mid_Top_Cup_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Vulc_A1_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Vulc_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Vulc_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Vulc_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Vulc_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Vulc_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Vulc_B_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Vulc_B_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Vulc_B_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_FT_Shoes_Vulc_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_B_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_B_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_B_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_C_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_C_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_C_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_C_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_No_Hair_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_No_Hair_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_No_Hair_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_No_Hair_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_No_Hair_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_No_Hair_B_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_No_Hair_B_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_No_Hair_B_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Cap_No_Hair_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Hair_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Hair_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Hair_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Hat_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_HD_Hat_No_Hair_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Jeans_A_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Jeans_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Jeans_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Jeans_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Jeans_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_Socks_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_Socks_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_Socks_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_Socks_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_Socks_A_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_Socks_A_07_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_Socks_A_08_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_Socks_A_09_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_Socks_A_10_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_Socks_A_11_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_LB_Shorts_Socks_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_Hoodie_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_Hoodie_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_Hoodie_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_Shirt_A1_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_Shirt_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_Shirt_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_TankTop_A1_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_TankTop_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_TankTop_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_TankTop_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_TankTop_A_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_Tshirt_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_Tshirt_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_GEN_UB_Tshirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Jenkem_Cap_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Jenkem_Cap_No_Hair_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Camo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Camo_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Camo_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Camo_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Camo_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Camo_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Camo_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Camo_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Cotton_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Cotton_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Cotton_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Cotton_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Cotton_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Cotton_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Cotton_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Cotton_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Cotton_ICON_7", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Cotton_ICON_8", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Denim_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Denim_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Denim_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Denim_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Denim_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Denim_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Denim_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Denim_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Denim_ICON_7", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_BaggyPants_Denim_ICON_8", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Camo_Denim_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Camo_Denim_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Camo_Denim_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Camo_Denim_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Camo_Denim_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Camo_Denim_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Camo_Denim_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Camo_Denim_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Cotton_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Cotton_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Cotton_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Cotton_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Cotton_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Cotton_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Cotton_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Cotton_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Cotton_ICON_7", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LB_GEN_Jeans_Cotton_ICON_8", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LimitedRun_Cap_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LimitedRun_Cap_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LimitedRun_Hoodie_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LimitedRun_Hoodie_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LimitedRun_Hoodie_c_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LimitedRun_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LimitedRun_TShirt_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_LimitedRun_TShirt_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Roger_TShirt_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Roger_TShirt_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Roger_TShirt_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Roger_TShirt_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Roger_TShirt_A_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Roger_TShirt_A_07_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Roger_TShirt_A_08_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Roger_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Schlaudie_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Shoes_Vul_A_eS_Square3_R_BlackGrey_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Shoes_Vul_A_eS_Square3_R_BlackGum_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Shoes_Vul_A_eS_Square3_R_BlackLeather_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Shoes_Vul_A_eS_Square3_R_BlackWhite_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Shoes_Vul_A_eS_Square3_R_Bone_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Shoes_Vul_A_eS_Square3_R_Brown_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Shoes_Vul_A_eS_Square3_R_Charcoal_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Shoes_Vul_A_eS_Square3_R_Green_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Shoes_Vul_A_eS_Square3_R_Navy_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Shoes_Vul_A_eS_Square3_R_Red_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Zero_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Zero_TShirt_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Zero_TShirt_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Zero_TShirt_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Zero_TShirt_E_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Zero_TShirt_F_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_Zero_TShirt_G_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ZeroHoodie_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AFXX_ZeroHoodie_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_Hoodie_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_Hoodie_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_Hoodie_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_Hoodie_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_Hoodie_E_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_Hoodie_F_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_TShirt_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_TShirt_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_TShirt_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_TShirt_E_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_TShirt_F_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ESP_TShirt_G_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_FlatFace_Hoodie_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_FlatFace_Hoodie_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_FlatFace_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_FlatFace_TShirt_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_FlatFace_TShirt_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_FlatFace_TShirt_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Adidas_3MCVulc_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_eSFootwear_AccelOG_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_eSFootwear_AccelOG_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_eSFootwear_AccelOG_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_eSFootwear_AccelOG_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_eSFootwear_AccelOG_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_eSFootwear_Explorer_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_eSFootwear_Sal23_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_eSFootwear_Sal23_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_eSFootwear_Sal23_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_eSFootwear_Sal23_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Globe_Mahalo01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_MidTop_Cup_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_MidTop_Cup_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_MidTop_Cup_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_MidTop_Cup_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_MidTop_Cup_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_MidTop_Cup_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_MidTop_Cup_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_Vul_A_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_Vul_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_Vul_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_Vul_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_Vul_B_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_Vul_B_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_Vul_B_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_Vul_B_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_FT_Shoes_Vul_B_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Beanie_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Beanie_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Beanie_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Beanie_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Beanie_A_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Beanie_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Beanie_NoHair_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Beanie_NoHair_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Beanie_NoHair_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Beanie_NoHair_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Beanie_NoHair_A_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Beanie_NoHair_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_A_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_A_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_B_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_B_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_B_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_B_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_B_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_B_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Cap_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_CapNo_Hair_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_CapNo_Hair_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_CapNo_Hair_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_CapNo_Hair_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_CapNo_Hair_A_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_CapNo_Hair_A_07_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_CapNo_Hair_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Hair_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Hair_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_HD_Pumpkin_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Hoodie_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Hoodie_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Hoodie_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Hoodie_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Hoodie_Debug_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Jacket_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Jacket_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_Jacket_Debug_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_Cargo_Pants_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_Cargo_Pants_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_Cargo_Pants_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_Cargo_Pants_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_Cargo_Pants_A_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_Cargo_Pants_A_07_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_Cargo_Pants_A_08_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_Cargo_Pants_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_NoComply_Cargo_Pants_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_StraightCut_Pants_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_StraightCut_Pants_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_StraightCut_Pants_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_StraightCut_Pants_A_Black_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_LB_StraightCut_Pants_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_TShirt_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_TShirt_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_TShirt_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_UB_Hoodie_A1_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_GEN_UB_Hoodie_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_HD_BEANIE_LiquidDeath_ThirstHunterSkullWarmer_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_HD_BEANIE_PlanB_Tab_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Head_Creature_Beanie_Prison_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Head_Creature_PandaHead", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Heroin_TShirt_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Heroin_TShirt_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Hoodie_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Jenkem_Cap_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Jenkem_Cap_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Jenkem_Cap_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Jenkem_TShirt_A_2_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Jenkem_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Camo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Camo_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Camo_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Camo_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Camo_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Camo_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Camo_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Camo_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Cotton_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Cotton_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Cotton_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Cotton_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Cotton_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Cotton_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Cotton_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Cotton_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Cotton_ICON_7", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Cotton_ICON_8", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Denim_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Denim_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Denim_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Denim_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Denim_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Denim_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Denim_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_BaggyPants_Denim_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_Shorts_Denim_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Denim_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Denim_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Denim_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Denim_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Denim_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Denim_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LB_GEN_StriaghtCut_Denim_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LimitedRun_Cap_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LimitedRun_Cap_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LimitedRun_Hoodie_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LimitedRun_Hoodie_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LimitedRun_Hoodie_c_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LimitedRun_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LimitedRun_TShirt_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_LimitedRun_TShirt_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_NoComply_TShirt_A_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_NoComply_TShirt_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_NoComply_TShirt_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_NoComply_TShirt_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_NoComply_TShirt_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_NoComply_TShirt_A_07_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_NoComply_TShirt_A_08_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_NoComply_TShirt_A_10_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_NoComply_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Roger_TShirt_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Roger_TShirt_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Roger_TShirt_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Roger_TShirt_A_05_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Roger_TShirt_A_06_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Roger_TShirt_A_07_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Roger_TShirt_A_08_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Schlaudie_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_HOOD_Almost_DoubleBar_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_HOOD_Almost_DoubleBar_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_HooideBaggy_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_ShirtBaggy_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TANK_Creature_Blank_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Almost_Intertwine_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Almost_Intertwine_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Antilanta_Matrix_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_BlackGold_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_BlackGold_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_BlackGold_B_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_BlackGold_B_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_BlackGold_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_BlackGold_C_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_BlackGold_C_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Blind_OgRippedSsPricePoint_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Blind_OgRippedSsPricePoint_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Blind_OgRippedSsPricePoint_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Blind_VintageBananaReaper_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Blind_VintageBananaReaper_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Blind_VintageBananaReaper_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Creature_Blank_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Creature_Prison_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_DC_BigLogo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_DC_BigLogo_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_DC_PocketT_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_DC_PocketT_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Jacuzzi_Flavour_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Jacuzzi_Flavour_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Jacuzzi_Koi_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Jacuzzi_Koi_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Jacuzzi_ScaredWeed_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Jacuzzi_ScaredWeed_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Satori_Rising_Sun_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Satori_Tea_Logo_2_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Satori_Tea_Logo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Schlaudie_LilLogo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Schlaudie_Pip_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Schlaudie_PirateShip_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Sugar_Gradient_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Sugar_Gradient_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_UB_TEE_Sugar_TieDye_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Zero_TShirt_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Zero_TShirt_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Zero_TShirt_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Zero_TShirt_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Zero_TShirt_E_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Zero_TShirt_F_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_Zero_TShirt_G_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ZeroHoodie_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_AMXX_ZeroHoodie_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_Costume_LiquidDeath", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_Costume_TMNT_Donatello_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_Costume_TMNT_Leo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_Costume_TMNT_Mikey_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_Costume_TMNT_Raph_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_Creature_Shoe_Prison_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_AllRed_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Almost_Greener_Black_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Almost_Greener_Green_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Almost_Most_Green_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Almost_Most_Red_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Almost_Most_Royal_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Almost_Scum_Punk_Blue_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Almost_Scum_Punk_Red_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Anatomy_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Antilanta_Bat_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Antilanta_Bat_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Antilanta_Car_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Antilanta_Footprints_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Antilanta_Kick_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Antilanta_Leafs_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Antilanta_Mask_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Antilanta_Matrix_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Antilanta_NBA_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Antilanta_Scribble_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_BigEgg_Pruple_Custom_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_BigEgg_Yellow_Custom_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_BlackEye_Custom_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Blind_OgRipped_Green_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Blind_OgRipped_Purple_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Blind_OgWallpaper_Blue_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Blind_RingerRedBlue_7000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Brandalised_BunnyThug_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Brandalised_FacemaskRatWithKnife_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Brandalised_FollowYourDreams_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Brandalised_GirlFloatingAway_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Brandalised_GraffitiIsACrime_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Brandalised_PandaGuns_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Brandalised_PinkGuardDog_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Brandalised_What_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_CityStars_Baller_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_CityStars_BlackType_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_CityStars_Colors_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_CityStars_DoloKareemCampbell_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_CityStars_LaType_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_CityStars_LegendKareemCampbell_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_CityStars_ReemosBailbondsKareemCampbell_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_CityStars_Stripes_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_CityStars_TAG_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_CraigScott_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_CraigScott_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Creature_Barbwire_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Creature_Police_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_CurbCrusher_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DeerMan_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Default_Blank725_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Default_Blank750_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Default_Blank775_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Default_Blank800_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Default_Blank8125_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Default_Blank8250_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Default_Blank875_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Default_UncutBoard_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Deviant_RibsManFiendClub_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Deviant_RibsManFiendClub_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Aztek_8010_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Bloom_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Connect_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_CrestBlue_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_CrestGreen_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Divine_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Friends_8025_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_GoodLuckSantos_8010_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Guadalupe_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_LoSide_8010_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Monogram_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Moonshine_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_OnIce_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Primo_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Quasar_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_RainbowDrip_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_RosraryLenticular_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_SlapUp_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Stix_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Sunshine_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_TrippinLenticular_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Tsunami_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Viper_8380_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_WetPaint_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_YingYang_8010_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DGK_Zodiac_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Diz_Army_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Diz_Hammer_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Diz_Speedo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Diz_TecMulticolor_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Diz_TecSkull_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Diz_TecSteel_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DLC_DLC_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_DMODW_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Donovan_CosmicDoom_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Element_AcAppleyardBbs_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Element_Blazin_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Element_DaydreamSection_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Element_Landrein_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Element_MagmaSeal_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Element_MoonDust_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Element_Seal_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Element_Square30_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_AllCapsBlack_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_AllCapsWhite_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_CatCollage_7000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_ClassicPandaBarletta_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_ClassicPandaSamarriaBlack_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_ClassicPandaSamarriaRhodamine_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_DontFit_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_Flipper_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_Funstick_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_HappyTree_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_JahPandaSamarria_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_JazzNightBarletta_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_JazzNightSamaria_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_KittenDreams_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_PeekabooProBandaBarletta_9500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_PlaidPandaSamaria_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_RainbowFartBlack_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_RainbowFartMint_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_RastaPanda_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_RenaissanceBarletta_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_SpectrumBlack_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_SpectrumWhite_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Enjoi_WhityPandaLogoWide_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_ESP_OgGraphic_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_ESP_Rockman_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_ESP_RockmanTile_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Faded_BYOB_8501_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_FlatFace_FlatFaceGreenFace_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_FlatFace_Font_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_FlatFace_OldFaces_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_BubbleLogoBlue_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_BubbleLogoOrange_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_HKD_Black_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_HKD_Blue_8400_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_HKD_Red_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_HKDBandanaBlack_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_HKDBandanaBlue_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_HKDBandanaRed_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_OdsseyStencilRed_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_OdsseyStencilYellow_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_OdysseyFaded_Black_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_OdysseyFaded_Ryl_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_OdysseyFaded_Yllw_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_QuattroOdysseyLumoBlack_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_QuattroOdysseyLumoBlue_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_ScriptBlack_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_ScriptBlue_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_SmokinTeamGreen8400_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_SmokinTeamRed8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_SmokinTeamYellow8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_TeamGlitchOrange_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_TeamGlitchTeal_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_TeamGltichPink_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_TeamSorryBlack_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_TeamSorryYellow_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_TeamStrobeBlue_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_TeamStrobeOrange_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_TeamTVlogoBlack8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Flip_TeamTVlogoBlue8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_FTP_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_FTP_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_FTP_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_FTP_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_FTP_F_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_FTP_G_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_FTP_H_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_FTP_Postal_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_FTP_Skulls_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_GreenEye_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_HighScores_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jacuzzi_500Years_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jacuzzi_BunnyLamp_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jacuzzi_Flavour_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jart_Akbar_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jart_Classic_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jart_Flagship_8375_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jart_Frame_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jart_Golden_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jart_Manny_Flag_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jart_Sculptural_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jart_Texture_8125_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Jart_Woodstock_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Kraken_01_BC1_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_LimitedRun_ForeverPhysical_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_LimitedRun_ForeverPhysicalBlack_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_LimitedRun_LimitedRun_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_DanielYeager_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_ExplosionBlack_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_ExplosionRed_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_ExplosionWhite_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_ExplosionYellow_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_MAI_DG_Madrid_HeshlordDezminLane_HeshlordShape_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_MAI_DG_Madrid_HeshlordDezminLane_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_TaroCardSeries_Lovers_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_TaroCardSeries_Reaper_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_WingsBlue_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_WingsMint_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_WingsPink_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Madrid_WingsYellow_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_BigPlantTeam_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_BigPlantTeam_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_BigPlantTeam_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_BigPlantTeam_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_BigPlantTeam_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_BrushTeam_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_BrushTeam_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_BrushTeam_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_BrushTeam_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_BrushTeam_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_BrushTeam_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_Medley_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_Station_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_WoodPlantTeam_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_WoodPlantTeam_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_WoodPlantTeam_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_WoodPlantTeam_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_WoodPlantTeam_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Magenta_WoodPlantTeam_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_AmyCaronBigfoot_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_AnnieGugliaBarto_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_AnnieGugliaWallieTunes_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_BigCat_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_BigCat_8000_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_BigCat_8000_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_BigCat_8000_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_BigCat_8000_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_CatMeowntain_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_Catzilla_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_Flamingo_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_Grandma_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_HalleysComet_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_Jazz_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_KristinEbelingKebs_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_Logo_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_Logo_8250_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_Logo_8250_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_Logo_8250_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_MariahDuranChaparral_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_MariahDuranWhiskers_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_MarmaladeTiger_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_Milo_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_OrigamiCat_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_Precarious_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_SamarriaBrevardDragonfly_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_SamarriaBrevardWelcomeHome_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_SpaceTiger_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_VanessaTorresFurreal_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Meow_VanessaTorresHeadspace_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Monarch_FountainLogo_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Monarch_FountainLogo_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Monarch_RialtoLogo_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Monarch_RialtoLogo_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Monarch_Synapse_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Monarch_Synapse_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_MovingCircus_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NBS_DazedStreak_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NBS_FaceYourGiants_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NBS_ItsTime_8100_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NBS_LoveOverHateBlack_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NBS_LoveOverHateWhite_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NBS_Musings_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NBS_NextLevel_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NBS_TenYearRose_8100_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NBS_TenYearSkull_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NoComply_Custom_Black_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NoComply_Custom_Blue_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NoComply_Custom_Green_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NoComply_Custom_Magenta_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_NoComply_Custom_Orange_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_OG_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_PipStreet_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_PlanB_90s_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_PlanB_BIGB_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_PlanB_EngrainedTeamDeck_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_PlanB_LewBlack_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_PlanB_LewBlue_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_PlanB_LewTeal_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_PlanB_LewWhite_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_PlanB_Primary_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_PlanB_TeamGrain_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_PlanB_Trill_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_RainDog_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_RazorEgg_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_RazorEgg_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_AustinBluebonnet_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_DogBalls_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_Doggo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_Doodler_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_Doodler_02_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_FirstPlace_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_FreeTheHand_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_GarrettGreens_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_GhostBoner_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_HalfRoger_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_HalfRoger_02_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_HalfRoger_03_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_HalfRogerOG_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_HalfRogerRight_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_HalfRogerStripes_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_HalfRogerWoodgrain_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_HeavyAndHappy_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_LangstonNeverMind_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_LangstonSkeleton_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_LoveRoger_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_MagicMirror_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_Max_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_Max_02_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_MaxSaturdayMorning_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_MeSoHorny_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_MTaylor_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_PinkLimo_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_PinkStarfish_8380_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_Reading_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_ReeseBeauty_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_ReeseMagnoliaTree_8380_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_RThompson_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_RThompson_02_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_Ryan_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_SausageParty_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_Scent_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_ScooterRoach_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_SeeYaLater_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_SkateRats_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_SkateSwitch_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_SkateSwitchPool_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_SockMonkey_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_SwordSwallower_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_Wallpaper_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_WeedCobra_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Roger_WhatIWant_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_RogerSkateCo_AustinAmelioSunset_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_RogerSkateCo_CurbCrusher_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_RogerSkateCo_GarrettYoungWaterfall_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_RogerSkateCo_RyanThompsonWaterSpirit_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_RoofTop_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SandlotTimes_HoppingFences_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SandlotTimes_Jaxxon_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SandlotTimes_NewHeights_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SandlotTimes_SandlotEast_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SandlotTimes_Skull_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SandlotTimes_SolosTaina_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SandlotTimes_Victoria_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SandlotTimes_WallMelter_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SantaCruz_ClassicDot_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SantaCruz_DressenRoses_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SantaCruz_KendallEndOfTheWorld_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SantaCruz_KnoxFirepit_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SantaCruz_MeekOGSlasher_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SantaCruz_SalbaTiger_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SantaCruz_ScreamingHand_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Schlaudie_AllRed_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Schlaudie_BirdEyeView_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Schlaudie_Constantine_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Schlaudie_Kraken_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Schlaudie_Military_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Schlaudie_MovingCircus_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Schlaudie_NiteOwl_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Schlaudie_PipStreet_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Schlaudie_ShipBattle_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Schlaudie_Spryte_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Schlaudie_TentaclesGreen_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Schlaudie_TentaclesPurple_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SessionOG_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Shovel_Custom_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SM_Jmoy_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SM_LesToasters_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SM_Mesa_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Spiral_Custom_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Suck_AcidRat_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Suck_AdultEnt_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Suck_AfterSex_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Suck_Botanical_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Suck_LadySmoker_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Suck_Spoon_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SugarSkateCo_80sChrome_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SugarSkateCo_BeachBall_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SugarSkateCo_Brain_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SugarSkateCo_Crayon_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SugarSkateCo_GinoGonzReissue_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SugarSkateCo_GoodVibes_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SugarSkateCo_HappyCloud_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SugarSkateCo_TypeScript_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_SugarSkateCo_ZanderMonster_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TheEgg_Custom_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TheEggxorcist_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TMNT_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TMNT_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TMNT_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TMNT_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TomDay_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_ToxicEye_800_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_ToxicEye_825_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_ToxicEye_CustomShape_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_AmericanMade_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_AnimalPudwill_7900_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_BadClouds_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_BluePlanetPudwill_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_CandyCloud_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_CollideLogo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_ColourBurst_8125_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_CoolerDaewon_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_DaewonRampage_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_DaewonTiki_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_Daytime_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_Essentials_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_EyeSpy_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_FlameOn_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_Freestyle_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_Fresh_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_FunkPaint_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_GeoBear_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_GeoTiger_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_GoneCuckoo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_GoodClouds_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_HeadInTheClouds_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_HeadInTheCoudsLightning_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_HealthNut_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_Howlin_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_JunkFood_8380_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_Medallion_BC_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_MindBlown_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_NightCloud_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_OrangeLogo_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_PinkLogo_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_Pops_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_RaisingHell_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_RampagePudwill_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_RedPlanet_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_Retro_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_Revenge_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_SayCheese_8000_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_SharkSnackPudwill_8500_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_Shroomcloud_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_SkullCloud_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_SleepyTimeTorey_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_SpotOnRed_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_SpotOnTeal_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_StoneageDaewon_8125_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_StoneagePudwill_7750_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_ToreyTiki_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_TortoiseToreyPudwill_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_TrillionDollarBaby_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_Undisputed_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_YeahBaby_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_TY_YellowLogo_8250_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_VeryBigEgg_Custom_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_CrazyTony_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_FaceOfALover_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_FairyTailPink_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_FairyTaleMiniTealBlack_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_GatewayTealDip_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_Hunny_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_InfinitelyBatty_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_Knight_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_Lamby2023_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_LambyOnAtheme_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_LightAndEasy_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_LilOwl_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_LooDoodBlack_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_LooDoodWhiteDip_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_MagildaLavenderTealStain_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_PeregrineBlackPurple_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_PeregrineNeonYellow_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_PeregrinePrism_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_PeregrineTealDip_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_SlothBlackLavender_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_SoilBone_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_SoilTealDip_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_SpecialEffects_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_Static2023_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_StaticBlack_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_TeddyLavenderDip_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_TeddyPrism_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_TrigerWhiteDip_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_TwentyEyesWhite_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_Unchained_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_VictimOfTimeBone_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_Widow_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Welcome_ZombieLoveBlack_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_3SkullBlood_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_American_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_BlackLight_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_BlackLightPanther_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_BloodSkull_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_BloodSkullGold_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_BurmanDevilleSkull_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_BurmanDickman_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_BurmanMummy_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_BurmanPastForms_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_ClassicBold_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_DamnItAllFont_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_DamnItAllVxFire_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_DaneBurman_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_DaneBurman_Incest_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_DeathRocker_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_LegacyRansom_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_Painkiller_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_Pharoah_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_PunkStarPink_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_Reaper_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_SingleBloodSkull_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_SingleSkull_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_Spiral_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_ZeroBlood_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zero_ZeroNumero_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DG_Zombie_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DL_Default_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DL_Default_01_MG_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DL_Default_02_BC1_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DL_Default_02_MG_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DL_Default_03_MG_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DL_Default_04_MG_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bone_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bone_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bone_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bone_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bone_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bone_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bone_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bone_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bone_ICON_7", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bulge_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bulge_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bulge_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bulge_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bulge_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bulge_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bulge_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bulge_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Bulge_ICON_7", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thick_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thick_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thick_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thick_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thick_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thick_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thick_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thick_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thick_ICON_7", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thin_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thin_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thin_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thin_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thin_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thin_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thin_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thin_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_DR_Thin_ICON_7", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Adidas_MatchBreakSuper_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Adidas_NorasShoe_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Adidas_NorasShoe_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Adidas_ProModel_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_DC_KalisVulc_BKW_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_DC_KalisVulc_XSSH_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_DC_OGLynx_GO6_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Ripper_Cole_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_Ripper_Cole_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_TheEasy_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_TheEasy_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_TheEasy_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_TheEasy_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_Fallen_TheEasy_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_MidTopCup_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_MidTopCup_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_MidTopCup_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_MidTopCup_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_MidTopCup_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_MidTopCup_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_MidTopCup_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_FT_MidTopCup_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Fallen_Claw_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Fallen_Claw_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Fallen_HendrixPatch_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Fallen_HendrixPatch_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Fallen_Trademark_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Fallen_Trademark_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Fallen_Trademark_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Fallen_Trademark_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_FTP_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_NBS_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_PlanB_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Suck_Ratcoste_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_HD_CAP_Suck_Ratcoste_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BeanieHat_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BeanieHat_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BeanieHat_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BeanieHat_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BeanieHat_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BeanieHat_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BeanieHat_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BeanieHat_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BucketHat_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BucketHat_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BucketHat_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BucketHat_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BucketHat_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BucketHat_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BucketHat_ICON_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_BucketHat_ICON_6", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_Diz_CircleSkull_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_Head_Cap_Diz_TecSkull_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_LB_Prison_Pants_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Brandalised_Crime_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Brandalised_FollowYourDreams_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Brandalised_Guard_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Brandalised_What_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Fallen_Insignia_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Fallen_Insignia_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Fallen_Insignia_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Fallen_RiseWith_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Fallen_RiseWith_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Fallen_TradeMark2_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Fallen_TradeMark2_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Fallen_TradeMark_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Fallen_TradeMark_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_Krux_KruxRainbow_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HOOD_OJ_Logo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_HooideBaggy_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_LongSleeve_Krux_KruxRainbow_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_LongSleeve_Krux_KruxRainbow_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_LS_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Almost_DoubleBar_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Almost_DoubleBar_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Brandalised_Bunny_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Brandalised_DreamBig_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Brandalised_Rat1_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Brandalised_Rat_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Destructo_Circle_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Destructo_Iron_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Destructo_Star_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_DLC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_DeadHead_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_DeadHead_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Desert_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Hendrix_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Hendrix_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Insignia_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Insignia_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Insignia_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Insignia_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_NightFiend_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Revolver_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Revolver_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_RiseWith_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_RiseWith_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Stripes_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Trademark_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Trademark_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Trademark_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Fallen_Trademark_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Krux_KruxRainbow_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_Krux_KruxRainbow_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_OJ_Logo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_OJ_Logo_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_SM_Colincoln_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_SM_LesToasters_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GEN_UB_TEE_SM_Mesa_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_BlackGold_FingerRing_Black_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_BlackGold_FingerRing_White_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_BlackGold_Magazin_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_BlackGold_OldEnglish_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Creature_Barbwire_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Creature_Bars_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Abstract_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Abstract_Painted_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_BigHorizontal_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Blue_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_BracketA_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_BracketA_Painted_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_DoubleLine_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_DoubleLine_Painted_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Geometric_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Geometric_Painted_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Green_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_LineHorizontal_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_LineHorizontal_Painted_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_LineVertical_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_LineVertical_Painted_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Orange_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Purple_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_QuadLines_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_QuadLines_Painted_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Red_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Session_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Session_Painted_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Spiral_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Spiral_Painted_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_StokedDivision_Cutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_StokedDivision_Painted_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Tape_Coutout_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Tape_Painted_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Default_Yellow_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_DGK_BackOf_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_DGK_Drippy_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_DGK_Floating_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_DGK_General_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_DGK_GlowUp_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_DGK_Guadalupe_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_DGK_Monogram_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_DGK_Primo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_DGK_Rosary_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_DGK_Serape_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_DGK_SmileNow_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Diz_SprayWings_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Flypaper_Diecut_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Flypaper_Swarm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Grizzly_GrizzlyTheBear_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Grizzly_GummyBears_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Grizzly_IceCreamTruck_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Grizzly_LapOfLuxury_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Grizzly_SittinOnChrome_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Grizzly_SnakeEyes_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Grizzly_SplitStamped_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Grizzly_StageDive_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Grizzly_StampPrint_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Grizzly_TiffanyStamp_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Grizzly_TigerKingOgBear_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Grizzly_ToTheMax_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Magenta_Bear_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Magenta_Camera_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Magenta_Fly_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Magenta_Swans_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_Magenta_VX_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_MOB_ArgyleBlue_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_MOB_BigLogo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_MOB_Palace_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_MOB_PinkStripes_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_MOB_Rainbow_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_MOB_RainbowBigLogo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_MOB_RainbowWallpaper_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_MOB_SmallLogo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_NBS_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_NBS_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_NoComply_Logo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_SessionOG_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_SM_ColinColn_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_SM_LesToasters_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_SM_Mesa_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_TMNT_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_TMNT_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_TMNT_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_TMNT_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_TY_DaeLa_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_GT_TY_ModernLogo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_NONE_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_REWARD_ALL_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_RP_1_16_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_RP_1_2_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_RP_1_4_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_RP_1_8_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_RP_3_8_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_SnowSkate_Deck_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_129_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_129_Mid_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_139_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_139_Mid_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_144_Mid_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_159_Mid_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_169_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_169_Mid_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_Debug_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_Default_01_BC_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_Default_01_MG_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_SessionOG_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_BP_SnowSkate_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_D1Avalanche_129_Low_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_D1Avalanche_139_Low_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_D1Raw_139_Mid_A_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_D1Raw_139_Mid_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_D1Raw_139_Mid_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_D1Raw_139_Mid_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_D1Raw_144_Mid_A_01_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_D1Raw_144_Mid_A_02_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_D1Raw_144_Mid_A_03_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_D1Raw_144_Mid_A_04_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Disruptor_139_Mid_A_Black_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Disruptor_139_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Disruptor_144_Mid_A_Black_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Disruptor_144_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Disruptor_149_Mid_A_Black_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Disruptor_149_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_GK6_144_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_HG_SnowSkate_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Kre-per_144_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Kre-per_149_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_KRUX_K5_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_KRUX_K5NoraTriangle_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Sendai_139_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Sendai_139_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Sendai_139_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Spark_144_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Spark_149_Mid_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Tensor_MagLight_ElemetsDaewon_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Tensor_MagLIght_NewGeometry_Gold_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Tensor_MagLIght_NewGeometry_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Tensor_MagLIght_NewGeometry_Red_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Tensor_MagLIght_NewGeometry_Rose_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Thunder_AppleyardExperience_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Thunder_Brevard_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Thunder_ColeBouquetLights_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_TK_Venture_Pudwill_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_Hood_Diz_Ind_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_Hood_Diz_Ind_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_Hood_Diz_TecSIgn_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_Hood_LiquidDeath_HydrateorDieRastaHoodie_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_Hood_Monarch_Atelier_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_Hood_Monarch_Atelier_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_Hood_Monarch_Atelier_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_HOOD_NBS_NextLevel_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_HOOD_NBS_NextLevel_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_HOOD_NBS_Ten_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_HOOD_NBS_Ten_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_Hood_PlanB_FullLogo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_Hood_PlanB_FullLogo_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_Hood_PlanB_FullLogo_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_Hood_Suck_Ratcoste_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_Hood_Suck_SuckItHard_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_HOOD_Zero_Ransom_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_DC_Logo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_DC_Logo_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_Diz_TechSign_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_LiquidDeath_InstantDeathSweatshirt_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_Monarch_Atelier_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_Monarch_Atelier_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_Monarch_Atelier_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_NBS_WeStillHere_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_NBS_WeStillHere_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_TMNT_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_TMNT_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_TMNT_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_TMNT_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_LS_TMNT_E_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_SWT_Faded_Logo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_SWT_iDabble_80s_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_SWT_iDabble_80s_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_SWT_PlanB_LogoCrew_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_SWT_PlanB_LogoCrew_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_SWT_Zero_ArmyTeam_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TBaggy_Monarch_Horus_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TBaggy_Monarch_Horus_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TBaggy_Monarch_Horus_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TBaggy_Monarch_Horus_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Diz_Army_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Diz_Ind_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Diz_Ind_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Diz_KeepItEvil_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Diz_NoSkull_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Diz_Script_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Diz_SparyWings_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Diz_SparyWings_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Diz_TecSign_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Faded_Logo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_FTP_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_FTP_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_FTP_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_LiquidDeath_ExclusiveDeathTee_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_LiquidDeath_HydrateorDieRedrumTee_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_CurveLogo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_CurveLogo_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_CurveLogo_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_CurveLogo_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_CurveLogo_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_CurveLogo_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_LifeBeginsHere_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_LifeBeginsHere_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_LifeBeginsHere_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_LifeBeginsHere_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_LifeBeginsHere_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_LifeBeginsHere_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_NeverGiveUp_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_NextLevel_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_NextLevel_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_NextLevel_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Support_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Support_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Support_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Support_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Support_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Support_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Ten_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Ten_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Ten_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Ten_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Ten_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Ten_ICON_4", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Wings_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_NBS_Wings_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_PlanB_FullLogoTee_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_PlanB_FullLogoTee_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_PlanB_FullLogoTee_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_PlanB_LogoLeft_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_PlanB_LogoLeft_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_PlanB_LogoLeft_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_PlanB_LogoTee_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_PlanB_LogoTee_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_PlanB_LogoTee_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_AcidRat_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_AcidRat_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_AfterSex_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_Botanical_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_HomeSweetHome_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_LadySmoker_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_LeRat_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_Logo_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_Logo_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_Spoon_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_Spoon_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_SuckIt_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Suck_SuckIt_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_TMNT_A_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_TMNT_B_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_TMNT_C_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_TMNT_D_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_TMNT_E_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_TMNT_F_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_TMNT_G_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_TMNT_H_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Zero_Army_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Zero_Army_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Zero_DamnItAll_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Zero_DamnItAllVX_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Zero_Ransom_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Zero_Skull_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Zero_Skull_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_UB_TEE_Zero_Stardust_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_ABS_Nexus_50_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_ABS_Nexus_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_ABS_Nexus_52_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_ABS_Nexus_52_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_ABS_Nexus_52_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_ABS_Nexus_52_ICON_3", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_ABS_Nexus_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_ABS_Nexus_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_ABXTorus_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_ABXTorus_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_DualUltraClassic_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_DualUltraClassic_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_DualUltraClassic_55_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_Evolution_51_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_Evolution_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_Evolution_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_GT1Wideboy_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_GT1Wideboy_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_StreetSlims_50_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_StreetSlims_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_TorusUltra_50_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_TorusUltra_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Autobahn_TorusUltra_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_50mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_50mm_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_50mm_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_RadialWide_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_RadialWide_50mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_RadialWide_50mm_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_RadialWide_50mm_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_Wide_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_Wide_50mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_Wide_50mm_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_Wide_50mm_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_Wide_Conic_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_Wide_Conic_50mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_Wide_Conic_50mm_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Default_TortoiseSeries_Wide_Conic_50mm_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_32mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_36mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_46mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_51mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_ConicWide_48mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_ConicWide_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_ConicWide_51mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_ConicWide_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_ConicWide_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_ConicWide_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_RadialWide_46mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_RadialWide_48mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_RadialWide_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_RadialWide_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_RadialWide_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_RadialWide_56mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Translucent_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Translucent_50mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Translucent_50mm_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Translucent_50mm_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Translucent_Wide_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Translucent_Wide_50mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Translucent_Wide_50mm_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Translucent_Wide_50mm_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Translucent_Wide_Conic_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Translucent_Wide_Conic_50mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Translucent_Wide_Conic_50mm_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Translucent_Wide_Conic_50mm_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Wide_48mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Wide_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Wide_51mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Wide_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Wide_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Classic_Wide_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Translucent_RadialWide_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Translucent_RadialWide_50mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Translucent_RadialWide_50mm_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Defualt_Translucent_RadialWide_50mm_ICON_2", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_101ChiChi_56_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_101PrimeCut_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_10thYear_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_DeathOnAcid_51_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_DeathOnAcid_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_DeathOnAcid_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_DeathOnAcid_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_Hazy_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_Lurk_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_Lurk_56_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_PrimeCut_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_PrimeCut_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_PrimeCut_55_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_Sneak_55_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Haze_Twirl_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_NBS_Classic52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_OJ_Erick_Winkowski_Tropics_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_OJ_JakeWootenScreamingCast_55_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_OJ_MiltonMartinezItsUsGold_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_OJ_NickBoserioLadybugs_60_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_OJ_NoraVasconcellosDreams_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_OJ_TeamLineNOMAD_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Ricta_Clouds_Black_56_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Ricta_Clouds_Red_57_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Ricta_Clouds_White_60_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Ricta_Impeller_Brevard_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Ricta_Naturals_Vortex_DesArmo_51_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Ricta_Sparx_MainFrame_Black_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Ricta_Speedrings_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Satori_BigLinkSeries_Classic_51mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Satori_DeLaSatori_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Satori_LilNuggs_GooBall_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Satori_LilNuggs_GooBall_54mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Satori_LinkedLogoTwoTone_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Satori_MeditationSeries_Classic_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Satori_Paisley_56_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Satori_RedEye_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Satori_TopShelfElephant_51_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Satori_TopShelfElephant_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Satori_TopShelfElephant_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Satori_TopShelfElephant_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SessionOG_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_AustynGilette_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_BackToThe90s_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_BubbsSeries_AustynGillette_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_BubbsSeries_MarkSuciu_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_ClassicSeries_AusytnsAlmonds_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_ClassicSeries_DrakeJones_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_ClassicSeries_JamesCraigOG_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_ClassicSeries_PianoHands_51mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_ClassicSeries_RaymondMolinar_Polaroid_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_CoffeeCruiser_50mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_CoffeeCruiser_50mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_CoffeeCruiser_50mm_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_CoffeeCruiser_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_CoffeeCruiser_52mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_CoffeeCruiser_52mm_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_CoffeeCruiser_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_CoffeeCruiser_54mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_CoffeeCruiser_54mm_ICON_1", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_CoffeeCruiser_56mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_DelicateSeries_AusytnGilette_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_DelicateSeries_MarkSuciu_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_DelicateSeries_SammyMontano_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_GroceryBag_51mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_GroceryBag_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_GroceryBag_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_GroceryBag_53mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_GroceryBag_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_GroceryBag_55mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_GroceryBag_56mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_Mark_Suciu_51mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_MaxGeronzi_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_MikeArnold_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_PencilPushersSeries_AustynGillette_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_PencilPushersSeries_ChrisJones_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_PencilPushersSeries_JoshPall_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_PencilPushersSeries_TomKnox_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_ReallysmlWheels_40mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_ReallysmlWheels_40mm_ICON_0", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_SoldticeSeries_AustynGillette_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_SoldticeSeries_TomKnox_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_SoldticeSeries_YounessAmrani_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_SucculentCruiser_DesertBlues_56mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_SucculentCruiser_Primes_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_Team_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_TheLoveSeries_MikeArnold_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_TheLoveSeries_SammyMontano_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_TheLoveSeries_TomKnox_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_Toonies_MikeArnold_54mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_Toonies_TomKnox_53mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_Toonies_YounessAmraniOG_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SML_YounessAmrani_52mm_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Snot_LilBoogers_48_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Snot_Team_51_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Snot_Team_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Snot_Team_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Snot_Team_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Snot_Team_55_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Snot_Team_56_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Snot_Team_58_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_SnowSkate_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_BigheadBurnPosse_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_BigheadPurpleYellow_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_BigheadWhiteGreen_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_BigheadWhitePurple_55_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_BigheadWhiteRed_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_ClassicBlue_56_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_ClassicGold_50_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_ClassicGray_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_ClassicGreen_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_ClassicOrange_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_ClassicPurple_58_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_ClassicRed_51_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_ClassicYellow_55_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_FormulaFour_56_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_FormulaFourEternals_56_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_NeonBigheadsBlue_57_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_NeonBigheadsGreen_53_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_NeonBigheadsPink_52_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_CIT_WL_Spitfire_NeonBigheadsYellow_54_ICON", RelativePath = $"Customization{Path.DirectorySeparatorChar}CITIcons" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "DATASS_CITMaterialSlotNames", RelativePath = $"Customization" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_BR_Default_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bearings{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_BR_Default_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Bearings{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Active_A_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Active{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Active_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Active{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Active_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Active" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Almost_Greener_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Almost_Greener_Green", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Almost_Most_Green", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Almost_Most_Red", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Almost_Most_Royal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Almost_Scum_Punk_Blue_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Almost_Scum_Punk_Red_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Almost_Greener_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Almost_Greener_Green", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Almost_Most_Green", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Almost_Most_Red", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Almost_Most_Royal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Almost_Scum_Punk_Blue_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Almost_Scum_Punk_Red_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Almost_Greener_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Almost_Greener_Green", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Almost_Most_Green", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Almost_Most_Red", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Almost_Most_Royal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Almost_Scum_Punk_Blue_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Almost_Scum_Punk_Red_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Almost{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Antilanta_GothamCity_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Antilanta_GothamCity_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Antilanta_NBA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Antilanta_Southernplayalistic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Antilanta_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Antilanta_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Antilanta_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Antilanta_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Antilanta_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Antilanta_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Antilanta_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Antilanta_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Antilanta_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Antilanta_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Anti_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Anti_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Anti_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Anti_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Anti_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Anti_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Anti_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Anti_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Anti_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Antilanta_BC_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Antilanta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Blind_OgRipped_Green_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Blind{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Blind_OgRipped_Purple_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Blind{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Blind_OgWallpaper_Blue_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Blind{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Blind_RingerRedBlue_7000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Blind{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Blind_OgRipped_Green_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Blind{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Blind_OgRipped_Purple_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Blind{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Blind_OgWallpaper_Blue_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Blind{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Blind_RingerRedBlue_7000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Blind{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Blind_OgRipped_Green_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Blind{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Blind_OgRipped_Purple_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Blind{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Blind_OgWallpaper_Blue_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Blind{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Blind_RingerRedBlue_7000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Blind{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Brandalised_BunnyThug", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Brandalised_FacemaskRatWithKnife", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Brandalised_FollowYourDreams", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Brandalised_GirlFloatingAway", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Brandalised_GraffitiIsACrime", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Brandalised_PandaGuns", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Brandalised_PinkGuardDog", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Brandalised_What", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Brandalised_BunnyThug", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Brandalised_FacemaskRatWithKnife", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Brandalised_FollowYourDreams", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Brandalised_GirlFloatingAway", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Brandalised_GraffitiIsACrime", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Brandalised_PandaGuns", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Brandalised_PinkGuardDog", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Brandalised_What", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Brandalised{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_CityStars_Baller", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_CityStars_BlackType", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_CityStars_Colors", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_CityStars_DoloKareemCampbell", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_CityStars_LaType", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_CityStars_LegendKareemCampbell", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_CityStars_ReemosBailbondsKareemCampbell", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_CityStars_Stripes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_CityStars_TAG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_CityStars_Baller", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_CityStars_BlackType", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_CityStars_Colors", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_CityStars_DoloKareemCampbell", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_CityStars_LaType", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_CityStars_LegendKareemCampbell", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_CityStars_ReemosBailbondsKareemCampbell", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_CityStars_Stripes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_CityStars_TAG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_CityStars_Baller", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_CityStars_BlackType", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_CityStars_Colors", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_CityStars_DoloKareemCampbell", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_CityStars_LaType", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_CityStars_LegendKareemCampbell", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_CityStars_ReemosBailbondsKareemCampbell", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_CityStars_Stripes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_CityStars_TAG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}CityStars{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Creature_Barbwire", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Creature_Police", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_DG__Creature_Police", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Creature_Barbewire", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SessionPolice", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Default_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Scratch", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Default_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Deviant_RibsManFiendClub_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Deviant{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Deviant_RibsManFiendClub_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Deviant{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Deviant_RibsManFiendClub_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Deviant{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Deviant_RibsManFiendClub_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Deviant{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Deviant_RibsManFiendClub", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Deviant{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Deviant_RibsManFiendClub_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Deviant{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Deviant_RibsManFiendClub_Top", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Deviant{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Divine", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Moonshine", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_OnIce", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Stix", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Zodiac", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_CrestBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_CrestGreen", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_Quasar", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_RosraryLenticular", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_DGK_TrippinLenticular", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Divine", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Moonshine", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_OnIce", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Stix", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Zodiac", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Aztek_8010", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Bloom_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Connect_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_CrestBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_CrestGreen", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Friends_8025", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_GoodLuckSantos_8010", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Guadalupe_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_LoSide_8010", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Monogram_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Primo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Quasar", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_RainbowDrip_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_RosraryLenticular", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_SlapUp_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Sunshine_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_TrippinLenticular", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Tsunami_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_Viper_8380", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_WetPaint_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_DGK_YingYang_8010", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Divine", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Moonshine", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_OnIce", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Stix", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Zodiac", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Aztek_8010", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Bloom_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Connect_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_CrestBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_CrestGreen", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Friends_8025", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_GoodLuckSantos_8010", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Guadalupe_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_LoSide_8010", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Monogram_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Primo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Quasar", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_RainbowDrip_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_RosaryLenticular_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_RosaryLenticular_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_SlapUp_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Sunshine_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_TirppinLenticular_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_TirppinLenticular_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Tsunami_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_Viper_8380", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_WetPaint_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_DGK_YingYang_8010", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Diz_Army", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Diz_Hammer", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Diz_Speedo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Diz_TecMulticolor", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Diz_TecSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Diz_TecSteel", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Diz_Army", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Diz_Hammer", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Diz_Speedo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Diz_TecMulticolor", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Diz_TecSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Diz_TecSteel", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Diz_Army", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Diz_Hammer", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Diz_Speedo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Diz_TecMulticolor", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Diz_TecSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Diz_TecSteel", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Donovan", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Donovan{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Donovan_CosmicDoom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Donovan{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Donovan_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Donovan{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Donovan_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Donovan{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Element_AcAppleyardBbs_8250_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Element_Blazin_8000_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Element_DaydreamSection_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Element_LandreinMA_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Element_MagmaSeal_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Element_MoonDust_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Element_Seal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Element_Square30Appleyard_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Element_AcAppleyardBbs", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Element_Blazin_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Element_DaydreamSection_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Element_LandreinMA_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Element_MagmaSeal_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Element_MoonDust_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Element_Seal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Element_Squared30Appleyard_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_AllCapsBlack_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_AllCapsWhite_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_CatCollage_7000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_ClassicPandaBarletta_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_ClassicPandaSamarriaBlack_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_ClassicPandaSamarriaRhodamine_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_DontFit_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_Flipper_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_Funstick_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_HappyTree_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_JahPandaSamarria_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_JazzNightBarletta_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_JazzNightSamaria_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_KittenDreams_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_PeekabooProBandaBarletta_9500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_PlaidPandaSamaria_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_RainbowFartBlack_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_RainbowFartMint_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_RastaPanda_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_RenaissanceBarletta_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_SpectrumBlack_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_SpectrumWhite_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Enjoi_WhityPandaLogoWide_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_AllCapsBlack_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_AllCapsWhite_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_CatCollage_7000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_ClassicPandaBarletta_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_ClassicPandaSamarriaBlack_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_ClassicPandaSamarriaRhodamine_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_DontFit_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_Flipper_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_Funstick_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_HappyTree_8500_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_JahPandaSamarria_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_JazzNightBarletta_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_JazzNightSamaria_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_KittenDreams_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_PeekabooProBandaBarletta_9500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_PlaidPandaSamaria_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_RainbowFartBlack_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_RainbowFartMint_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_RastaPanda_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_RenaissanceBarletta_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_SpectrumBlack_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_SpectrumWhite_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Enjoi_WhityPandaLogoWide_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Enjoi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_ESP_Rockman", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_ESP_RockmanTile", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_ESP_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_ESP_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_ESP_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ESP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Faded_BYOB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Faded{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Faded_BYOB_8500_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Faded{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Faded_Friday_8500_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Faded{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_FlatFace_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_FlatFace_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_FlatFace_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_FlatFace_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_FlatFace_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_FlatFace_03_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FlatFace{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_HKD_Black_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_HKD_Blue_8400", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_HKD_Red_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_HKDBandanaBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_HKDBandanaBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_HKDBandanaRed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_OdysseyFaded_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_OdysseyFaded_Ryl", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_OdysseyFaded_Yllw", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_SmokinTeamGreen8400", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_SmokinTeamRed8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_SmokinTeamYellow8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_TeamGlitchOrange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_TeamGlitchTeal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_TeamGltichPink", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_TeamTVlogoBlack8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Flip_TeamTVlogoBlue8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_HKD_Black_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_HKD_Blue_8400", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_HKD_Red_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_HKDBandanaBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_HKDBandanaBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_HKDBandanaRed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_OdysseyFaded_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_OdysseyFaded_Ryl", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_OdysseyFaded_Yllw", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_SmokinTeamGreen8400", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_SmokinTeamRed8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_SmokinTeamYellow8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_TeamGlitchOrange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_TeamGlitchTeal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_TeamGltichPink", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_TeamTVlogoBlack8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_TeamTVlogoBlue8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_BubbleLogoBlue_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAi_DG_Flip_BubbleLogoOrange_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_OdsseyStencilRed_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_OdsseyStencilYellow_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_QuattroOdysseyLumo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_ScriptBlack_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_ScriptBlue_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_TeamSorryBlack_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_TeamSorryYellow_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_TeamStrobeBlue_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Flip_TeamStrobeOrange_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_HKD_Black_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_HKD_Blue_8400", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_HKD_Red_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_HKDBandanaBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_HKDBandanaRed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_HKDBandanaRedBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_OdysseyFaded_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_OdysseyFaded_Ryl", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_OdysseyFaded_Yllw", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_SmokinTeamGreen8400", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_SmokinTeamRed8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_SmokinTeamYellow8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_TeamGlitchOrange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_TeamGlitchTeal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_TeamGltichPink", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_TeamTVlogoBlack8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_TeamTVlogoBlue8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_BubbleLogoBlue_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_BubbleLogoOrange_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_OdsseyStencilRed_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_OdsseyStencilYellow_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_QuattroOdysseyLumo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_ScriptBlack_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_ScriptBlue_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_TeamSorryBlack_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_TeamSorryYellow_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_TeamStrobeBlue_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Flip_TeamStrobeOrange_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_FTP_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_FTP_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_FTP_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_FTP_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_FTP_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_FTP_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_FTP_Postal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_FTP_Skulls", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_FTP_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_FTP_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_FTP_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_FTP_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_FTP_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_FTP_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_FTP_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_FTP_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_FTP_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_FTP_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_FTP_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_FTP_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_FTP_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_FTP_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_FTP_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_FTP_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}FTP{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_CurbCrusher_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_CurbCrusher_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Anatomy_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Anatomy_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BigEgg_Yellow_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BigEgg_Yellow_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BlackEye_01_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BlackEye_01_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_DeerMan_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_DeerMan_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Fish_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Fish_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Shovel_01_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Shovel_01_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Spiral_01_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Spiral_01_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_TheEgg_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_TheEgg_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_VeryBigEgg_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_VeryBigEgg_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_CurbCrusher_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_CurbCrusher_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Anatomy_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Anatomy_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BigEgg_Yellow_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BigEgg_Yellow_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BlackEye_01_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BlackEye_01_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_DeerMan_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_DeerMan_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Fish_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Fish_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Shovel_01_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Shovel_01_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Spiral_01_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Spiral_01_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_TheEgg_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_TheEgg_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_VeryBigEgg_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_VeryBigEgg_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_CurbCrusher_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_CurbCrusher_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Anatomy_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Anatomy_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BigEgg_Yellow_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BigEgg_Yellow_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BlackEye_01_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BlackEye_01_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_DeerMan_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_DeerMan_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Fish_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Fish_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Shovel_01_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Shovel_01_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Spiral_01_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Spiral_01_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_TheEgg_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_TheEgg_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_VeryBigEgg_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_VeryBigEgg_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_CurbCrusher", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Anatomy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BigEgg_Yellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_BlackEye_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_DeerMan", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Fish", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Shovel_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_Spiral_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_TheEgg", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Heroin_Deck_VeryBigEgg", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_RainDog", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Muted" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_Shovel_Custom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Muted" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Heroin_Zombie", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}Muted" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_07", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_08", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_09", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_10", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_11", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_12", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_13", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_14", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_15", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_16", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_17", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_18", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_19", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_Anatomy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_Eyes_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_Eyes_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_Eyes_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_Eyes_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Heroin_VeryBigEgg", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_Anatomy_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_BlackEye_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_CraigScott_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_CraigScott_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_CurbCrusher_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_DeerMan_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_DMODW_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_Eye1_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_Eye2_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_Eye3_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_FrankShaw_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_GreenEye_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_HighScores_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_RainDog_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_RazorEgg_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_RazorEgg_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_ShovelHead_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_Spiral_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_TheEgg_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_TheEgg_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_TheEggxorcist_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_TomDay_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_VeryBigEgg_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Heroin_Zombie_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Heroin{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jacuzzi_500Years", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jacuzzi_BunnyLamp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Jacuzzi_Flavour", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jacuzzi_500Years", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jacuzzi_BunnyLamp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jacuzzi_Flavour", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jacuzzi_500Years", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jacuzzi_BunnyLamp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jacuzzi_Flavour", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jart_Akbar_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jart_Classic_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jart_Flagship_8375", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jart_Frame_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jart_Golden_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jart_Manny_Flag_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jart_sculptural_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jart_texture_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Jart_Woodstock_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jart_Akbar_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jart_Classic_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jart_Flagship_8_375", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jart_Frame_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jart_Golden_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jart_Manny_Flag_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jart_sculptural_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jart_texture_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Jart_Woodstock_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}JartSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_LimitedRun_ForeverPhysical", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_LimitedRun_ForeverPhysicalBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_LimitedRun_LimitedRun", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_LimitedRun_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_LimitedRun_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_LimitedRun_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_LimitedRun_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_LimitedRun_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_LimitedRun_03_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}LimitedRun{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Madrid_DanielYeager", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Madrid_ExplosionBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Madrid_ExplosionRed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Madrid_ExplosionWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Madrid_ExplosionYellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Madrid_HeshlordDezminLane", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Madrid_TaroCardSeries_Lovers", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Madrid_TaroCardSeries_Reaper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Madrid_WingsBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Madrid_WingsMint", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Madrid_WingsPink", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Madrid_WingsYellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Madrid_DanielYeager_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Madrid_ExplosionBlack_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Madrid_ExplosionRed_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Madrid_ExplosionWhite_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Madrid_ExplosionYellow_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Madrid_Heshlord_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Madrid_Lovers_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Madrid_Reaper_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Madrid_WingsBlue_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Madrid_WingsMint_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Madrid_WingsPink_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Madrid_WingsYellow_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MadridSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Magenta_BigPlantTeam", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Magenta_BrushTeam", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Magenta_Medley", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Magenta_Station", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Magenta_WoodPlantTeam", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Magenta_BigPlantTeam", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Magenta_BrushTeam", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Magenta_Medley", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Magenta_Station", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Magenta_WoodPlantTeam", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MagentaSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_AmyCaronBigfoot", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_AnnieGugliaBarto", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_KristinEbelingKebs", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_MariahDuranChaparral", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_MariahDuranWhiskers", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_SamarriaBrevardDragonfly", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_SamarriaBrevardWelcomeHome", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_VanessaTorresFurreal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Meow_VanessaTorresHeadspace", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_AmyCaronBigfoot", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_AnnieGugliaBarto", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_KristinEbelingKebs", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_MariahDuranChaparral", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_MariahDuranWhiskers", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_SamarriaBrevardDragonfly", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_SamarriaBrevardWelcomeHome", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_VanessaTorresFurreal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_VanessaTorresHeadspace", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_AnnieGugliaWallieTunes_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_BigCatGreen_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_BigCatPink_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_BigCatPurple_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_BigCatTeal_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_BigCatWhite_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_CatMeowntain_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_Catzilla_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_Flamingo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_Grandma_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_HalleysComet_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_Jazz_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_LogoGreen_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_LogoPink_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_LogoPurple_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_LogoTeal_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_MarmaladeTiger_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_Milo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_OrigamiCat_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_Precarious_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Meow_SpaceTiger_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_AmyCaronBigfoot", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_AnnieGugliaBarto", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_KristinEbelingKebs", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_MariahDuranChaparral", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_MariahDuranWhiskers", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_SamarriaBrevardDragonfly", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_SamarriaBrevardWelcomeHome", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_VanessaTorresFurreal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_VanessaTorresHeadspace", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_AnnieGugliaWallieTunes_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_BigCatGreen_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_BigCatPink_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_BigCatPurple_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_BigCatTeal_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_BigCatWhite_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_CatMeowntain_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_Catzilla_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_Flamingo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_Grandma_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_HalleysComet_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_Jazz_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_LogoGreen_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_LogoPink_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_LogoPurple_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_LogoTeal_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_MarmaladeTiger_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_Milo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_OrigamiCat_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_Precarious_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Meow_SpaceTiger_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}MeowSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Monarch_FountainLogo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Monarch_FountainLogo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Monarch_RialtoLogo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Monarch_RialtoLogo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Monarch_Synapse_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Monarch_Synapse_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Monarch_FountainLogo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Monarch_FountainLogo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Monarch_RialtoLogo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Monarch_RialtoLogo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Monarch_Synapse_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Monarch_Synapse_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Monarch_FountainLogo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Monarch_FountainLogo_A_CutMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Monarch_FountainLogo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Monarch_RialtoLogo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Monarch_RialtoLogo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Monarch_Synapse_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Monarch_Synapse_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Monarch{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NBS_DazedStreak_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NBS_FaceYourGiants_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NBS_ItsTime_8100", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NBS_LoveOverHateBlack_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NBS_LoveOverHateWhite_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NBS_Musings_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NBS_NextLevel_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NBS_TenYearRose_8100", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_NBS_TenYearSkull_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_NBS_DazedStreak_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_NBS_FaceYourGiants_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_NBS_ItsTime_8100", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_NBS_LoveOverHateBlack_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_NBS_LoveOverHateWhite_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_NBS_Musings_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_NBS_NextLevel_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_NBS_TenYearRose_8100", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_NBS_TenYearSkull_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_NBS_DazedStreak_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_NBS_FaceYourGiants_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_NBS_ItsTime_8100", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_NBS_LoveOverHateBlack_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_NBS_LoveOverHateWhite_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_NBS_Musings_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_NBS_NextLevel_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_NBS_TenYearRose_8100", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_NBS_TenYearSkull_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_NoComplyLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_NoComply_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_NoComply_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_NoComplyLogo_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_PlanB_90s", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_PlanB_BIGB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_PlanB_EngrainedTeamDeck", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_PlanB_LewBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_PlanB_LewBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_PlanB_LewTeal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_PlanB_LewWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_PlanB_Primary", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_PlanB_TeamGrain", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_PlanB_Trill", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_PlanB_90s", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_PlanB_BIGB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_PlanB_EngrainedTeamDeck", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_PlanB_LewBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_PlanB_LewBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_PlanB_LewTeal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_PlanB_LewWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_PlanB_Primary", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_PlanB_TeamGrain", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_PlanB_Trill", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_PlanB_90s", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_PlanB_BIGB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_PlanB_EngrainedTeamDeck", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_PlanB_LewBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_PlanB_LewBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_PlanB_LewTeal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_PlanB_LewWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_PlanB_Primary", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_PlanB_TeamGrain", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_PlanB_Trill", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_PlanB_Trill_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_AustinBluebonnet", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_Doggo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_GarrettGreens", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_HalfRogerOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_HalfRogerStripes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_HalfRogerWoodgrain", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_HeavyAndHappy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_LangstonSkeleton", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_LoveRoger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_MagicMirror", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_MaxSaturdayMorning", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_ReeseBeauty", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_SeeYaLater", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Roger_SkateSwitchPool", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_RogerSkateCo_AustinAmelioSunset", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_RogerSkateCo_CurbCrusher", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_RogerSkateCo_GarrettYoungWaterfall", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_RogerSkateCo_RyanThompsonWaterSpirit", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_AustinBluebonnet", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_Doggo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_GarrettGreens", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_HalfRogerOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_HalfRogerStripes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_HalfRogerWoodgrain", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_HeavyAndHappy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_LangstonSkeleton", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_LoveRoger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_MagicMirror", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_MaxSaturdayMorning", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_ReeseBeauty", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_SeeYaLater", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Roger_SkateSwitchPool", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_07", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_08", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_09", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_10", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_11", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_12", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_13", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_14", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_15", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_16", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_17", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_18", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_19", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_20", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_21", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_22", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_23", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_24", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_25", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_26", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_27", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_AustinAmelioSunset", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_CurbCrusher", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_GarrettYoungWaterfall", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_LangstonNeverMind", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_PinkStarfish", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_ReeseMagnoliaTree", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_RyanThompsonWaterSpirit", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_ScooterRoach", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_RogerSkateCo_Wallpaper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_AustinBluebonnet", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_Doggo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_GarrettGreens", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_HalfRogerOG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_HalfRogerStripes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_HalfRogerWoodgrain", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_HeavyAndHappy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_LangstonSkeleton", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_LoveRoger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_MagicMirror", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_MaxSaturdayMorning", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_ReeseBeauty", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_SeeYaLater", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_SkateSwitchPool", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}2024Feb" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_GhostBoner_01_Scratched_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_DogBalls_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_Doodler_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_Doodler_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_FirstPlace_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_FreeTheHand_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_GhostBoner_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_HalfRoger_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_HalfRoger_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_HalfRoger_03_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_HalfRoger_04_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_MaxTaylor_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_MeSoHorny_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_MTaylor_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_PinkLimo_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_Reading_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_RThompson_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_RyanThompson_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_RyanThompson_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_SausageParty_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_Scent_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_SkateRats_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_SkateSwitch_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_SockMonkey_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_SwordSwallower_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_WeedCobra_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Roger_WhatIWant_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_RogerSkateCo_AustinAmelioSunset", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_RogerSkateCo_CurbCrusher", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_RogerSkateCo_GarrettYoungWaterfall", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_RogerSkateCo_LangstonNeverMind", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_RogerSkateCo_PinkStarfish", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_RogerSkateCo_ReeseMagnoliaTree", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_RogerSkateCo_RyanThompsonWaterSpirit", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_RogerSkateCo_ScooterRoach", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_RogerSkateCo_Wallpaper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}RogerSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SandlotTimes_HoppingFences", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SandlotTimes_NewHeights", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SandlotTimes_Skull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SandlotTimes_SolosTaina", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SandlotTimes_Victoria", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SandlotTimes_WallMelter", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SandlotTimes_HoppingFences", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SandlotTimes_NewHeights", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SandlotTimes_Skull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SandlotTimes_SolosTaina", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SandlotTimes_Victoria", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SandlotTimes_WallMelter", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SandlotTimes_HoppingFences", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SandlotTimes_NewHeights", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SandlotTimes_Skull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SandlotTimes_SolosTaina", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SandlotTimes_Victoria", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SandlotTimes_WallMelter", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_SantaCruz_SC175", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}BaseMesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SantaCruz_ClassicDot_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SantaCruz_DressenRoses", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SantaCruz_KendallEndOfTheWorld", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SantaCruz_KnoxFirepit", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SantaCruz_MeekOGSlasher", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SantaCruz_SalbaTiger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SantaCruz_ScreamingHand_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SantaCruz_ClassicDot", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SantaCruz_KendallEndOfTheWorld", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SantaCruz_MeekOGSlasher", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SantaCruz_SalbaTiger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SantaCruz_ScreamingHand_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WT_SantaCruz_DressenRoses", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Schlaudie_AllRed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Schlaudie_BirdEyeView", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Schlaudie_Constantine", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Schlaudie_Kraken", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Schlaudie_Military", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Schlaudie_MovingCircus", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Schlaudie_NiteOwl", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Schlaudie_PipStreet", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Schlaudie_ShipBattle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Schlaudie_Spryte", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Schlaudie_TentaclesGreen", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Schlaudie_TentaclesPurple", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Schlaudie_BirdEyeView", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Schlaudie_GreenTentacles", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Schlaudie_Letter", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Schlaudie_TreasureMap", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Tentacles_Top_Green_10x35__top_", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_MovingCircus_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_PipStreet_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_BirdEyeView_TOP_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_GreenTentacles_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_Kraken_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_Logo_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_Military_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_Military_TOP_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_NiteOwl_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_PurpleTentacles_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_RedLogo_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_Rooftop_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_Rooftop_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_ShipBattle_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Schlaudie_TreasureMap_TOP_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SchlaudieSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SM_Jmoy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SM_LesToasters", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_SM_Mesa", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SM_Jmoy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SM_LesToasters", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SM_Mesa", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SM_Jmoy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SM_LesToasters", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SM_Mesa", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Suck_AcidRat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Suck_AdultEnt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Suck_Botanical", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Suck_AcidRat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Suck_AdultEnt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Suck_AfterSex", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Suck_Botanical", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Suck_LadySmoker", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Suck_Spoon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Suck_AcidRat", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Suck_AdultEnt", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Suck_AfterSex", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Suck_Botanical", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Suck_LadySmoker", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Suck_Spoon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Suck{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SugarSkateCo_80sChrome", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SugarSkateCo_BeachBall", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SugarSkateCo_Brain", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SugarSkateCo_Crayon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SugarSkateCo_GinoGonzReissue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SugarSkateCo_GoodVibes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SugarSkateCo_HappyCloud", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SugarSkateCo_TypeScript", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_SugarSkateCo_ZanderMonster", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_SugarSkateCo_80sChrome_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_SugarSkateCo_BeachBall_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_SugarSkateCo_Brain_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_SugarSkateCo_Crayon_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_SugarSkateCo_GinoGonzReissue_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_SugarSkateCo_GoodVibes_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_SugarSkateCo_HappyCloud_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_SugarSkateCo_TypeScript_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_SugarSkateCo_ZanderMonster_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}SugarSkateCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_AmericanMade", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_Daytime", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_Freestyle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_FunkPaint", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_GoneCuckoo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_HeadInTheCoudsLightning", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_MindBlown", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_Pops", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_RaisingHell", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_Retro", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_Shroomcloud", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_SleepyTimeTorey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_TortoiseToreyPudwill", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_TrillionDollarBaby", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_Undisputed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_YeahBaby", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_CandyCloud", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_CollideLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_DaewonTiki", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_Fresh", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_GeoBear", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_GeoTiger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_SkullCloud", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_StinkerPudwill_RBSticker_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TY_ToreyTiki", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_AmericanMade", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_Daytime", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_Freestyle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_FunkPaint", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_GoneCuckoo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_HeadInTheCoudsLightning", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_MindBlown", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_Pops", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_RaisingHell", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_Retro", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_Shroomcloud", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_SleepyTimeTorey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_TortoiseToreyPudwill", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_TrillionDollarBaby", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_Undisputed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_YeahBaby", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_AnimalPudwill_7900", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_BadClouds_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_BluePlanetPudwill_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_CandyCloud", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_CollideLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_ColourBurst_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_CoolerDaewon_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_DaewonRampage_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_DaewonTiki", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_Essentials_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_EyeSpy_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_FlameOn_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_Fresh", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_GeoBear", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_GeoTiger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_GoodClouds_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_HeadInTheClouds_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_HealthNut_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_Howlin_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_JunkFood_8380", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_Medallion_BC_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_NightCloud_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_OrangeLogo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_PinkLogo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_RampagePudwill_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_RedPlanet_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_Revenge_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_SayCheese_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_SharkSnackPudwill_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_SkullCloud", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_SpotOnRed_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_SpotOnTeal_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_StinkerPudwill_RBSticker_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_StoneageDaewon_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_StoneageDaewon_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_ToreyTiki", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TY_YellowLogo_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_AmericanMade", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_Daytime", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_Freestyle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_FunkPaint", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_GoneCuckoo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_HeadInTheCoudsLightning", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_MindBlown", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_Pops", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_RaisingHell", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_Retro", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_Shroomcloud", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_SleepyTimeTorey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_TortoiseToreyPudwill", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_TrillionDollarBaby", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_Undisputed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_YeahBaby", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_ThankYou_CandyCloud", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_ThankYou_CollideLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_ThankYou_DaewonTiki", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_ThankYou_Fresh", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_ThankYou_GeoBear", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_ThankYou_GeoTiger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_ThankYou_SkullCloud", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_ThankYou_ToreyTiki", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_AnimalPudwill_7900", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_BadClouds_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_BluePlanetPudwill_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_ColourBurst_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_CoolerDaewon_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_DaewonRampage_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_Essentials_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_EyeSpy_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_FlameOn_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_GoodClouds_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_HeadInTheClouds_82500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_HealthNut_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_Howlin_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_JunkFood_8380", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_Medallion_BC_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_NightCloud_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_OrangeLogo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_PinkLogo_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_RampagePudwill_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_RedPlanet_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_Revenge_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_SarkSnackPudwill_8500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_SayCheese_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_SpotOnRed_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_SpotOnTeal_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_StinkerPudwill_RBSticker_8000", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_StoneageDaewon_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_StoneageDaewon_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TY_YellowLogo_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TMNT_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TMNT_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TMNT_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_TMNT_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TMNT_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TMNT_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TMNT_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_TMNT_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TMNT_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TMNT_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TMNT_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_TMNT_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_CrazyTony", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_FaceOfALover", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_Hunny", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_InfinitelyBatty", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_Knight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_Lamby2023", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_LightAndEasy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_LilOwl", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_SpecialEffects", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_Static2023", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_Unchained", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_Widow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_StaticBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_TrigerWhiteDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_VictimOfTimeBone", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Welcome_ZombieLoveBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_CrazyTony", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_FaceOfALover", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_Hunny", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_InfinitelyBatty", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_Knight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_Lamby", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_LightAndEasy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_LilOwl", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_SpecialEffects", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_Static", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_Unchained", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_Widow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_FairyTailPink", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_FairyTaleMiniTealBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_GatewayTealDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_LambyOnAtheme", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_LooDoodBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_LooDoodWhiteDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_MagildaLavenderTealStain", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_PeregrineBlackPurple", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_PeregrineNeonYellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_PeregrinePrism", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_PeregrineTealDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_SlothBlackLavender", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_SoilBone", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_SoilTealDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_StaticBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_TeddyLavenderDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_TeddyPrism", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_TrigerWhiteDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_TwentyEyesWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_VictimOfTimeBone", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Welcome_ZombieLoveBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_CrazyTony", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_FaceOfALover", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_Hunny", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_InfinitelyBatty", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_Knight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_Lamby", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_LightAndEasy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_LilOwl", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_SpecialEffects", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_Static", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_Unchained", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_Widow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}BatchMar2023" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_FairyTailPink", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_FairyTaleMiniTealBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_GatewayTealDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_LambyOnAtheme", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_LooDoodBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_LooDoodWhiteDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_MagildaLavenderTealStain", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_PeregrineBlackPurple", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_PeregrineNeonYellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_PeregrinePrism", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_PeregrineTealDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_SlothBlackLavender", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_SoilBone", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_SoilTealDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_StaticBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_TeddyLavenderDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_TeddyPrism", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_TrigerWhiteDip", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_TwentyEyesWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_VictimOfTimeBone", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Welcome_ZombieLoveBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_BlackLight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_BlackLightPanther", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_DeathRocker", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_Pharoah", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DG_Zero_Reaper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_07", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_08", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_09", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_10", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_11", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_12", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_13", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_14", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_15", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_16", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_17", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_18", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_3SkullBlood", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_BlackLight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_BlackLightPanther", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_BloodSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_BurmanMummy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_DeathRocker", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_Painkiller", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_Pharoah", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DG_Zero_Reaper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_BlackLight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Cole" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_BlackLightPanther", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Cole" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_DeathRocker", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Cole" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_Pharoah", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Cole" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_Reaper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures{Path.DirectorySeparatorChar}Cole" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_3Skull_Blood_Deck_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_3Skull_BloodRed_Deck_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_American_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_BloodSkull", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_Burman_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_Burman_DevilleSkull_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_Burman_Mummy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_Burman_PastForms_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_DamnItAll_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_DamnItAll_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_Dickman_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_InsectArt_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_LegacyRansom_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_PainKiller", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_PunkStarPink_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_Sessions_Numero_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_SingleBloodSkull_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_SingleBloodSkull_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_Spiral_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_ZeroBlood_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_Zero_ZeroBold_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckGraphics{Path.DirectorySeparatorChar}ZeroSkateboards{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DL_Default_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DL_Default_02_BC1", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DL_Default_02_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DL_Default_03_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_DL_Default_04_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "M_DeckLayersFullDip_Black_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "M_DeckLayersFullDip_Blue_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "M_DeckLayersFullDip_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "M_DeckLayersFullDip_Teal_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "M_DeckLayersFullDip_White_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DL_Default_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DL_Default_03_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DL_Default_04_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DL_Default_05_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DL_Default_06_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckLayers{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Rail_Default_01_Broken_nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BrokenMiddle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Rail_Default_01_Broken_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BrokenMiddle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Rail_Default_02_Broken_nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BrokenMiddle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Rail_Default_02_Broken_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BrokenMiddle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Rail_Default_03_Broken_nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BrokenMiddle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Rail_Default_03_Broken_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BrokenMiddle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Rail_Default_04_Broken_nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BrokenMiddle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Rail_Default_04_Broken_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BrokenMiddle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DeckRail_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DeckRail_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DeckRail_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DeckRail_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DeckRail_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DeckRail_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DeckRail_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_DeckRail_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}DeckRails{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_AMXX_Madrid_Heshlord_8600_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_AMXX_Madrid_Heshlord_8600_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_AMXX_Madrid_Heshlord_8600_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_AMXX_Madrid_Heshlord_8600_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_AMXX_Madrid_Heshlord_8600_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_AMXX_Madrid_Heshlord_8600_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_AMXX_Madrid_Heshlord_8600", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Madrid" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_A_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_A_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_B_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_B_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_C_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_C_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_D_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_D_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_E_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_E_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_F_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_F_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_G_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_G_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_H_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_H_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Middle" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_A_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_A_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_B_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_B_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_C_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_C_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_D_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_D_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_E_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_E_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_F_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_F_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Nose" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_A_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_A_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_B_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_B_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_C_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_C_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_D_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_D_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_E_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_E_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_F_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_F_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Broken{Path.DirectorySeparatorChar}Tail" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Welcome_Custom_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Welcome" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_BoardBreakMiddelLayers", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_7250_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_7250_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_7500_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_7500_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_7750_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_7750_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8000_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8000_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8125_A_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8125_A_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8125_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8125_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8250_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8250_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8500_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8500_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8500_BrokenNose_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8500_BrokenNose_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8500_BrokenTail_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8500_BrokenTail_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8750_BrokenMiddle_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8750_BrokenMiddle_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_TempBrokenBoard_Nose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_TempBrokenBoard_Tail", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh{Path.DirectorySeparatorChar}BoardBreaking" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_7250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_7500", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_7750", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_8125", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Deck_Default_UncutBoard", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Decks{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_BlackGold_FingerRing_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}BlackGold{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_BlackGold_FingerRing_White", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}BlackGold{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_BlackGold_Magazin", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}BlackGold{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_BlackGold_OldEnglish", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}BlackGold{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_BlackGold_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}BlackGold{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_BlackGold_BA", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}BlackGold{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_BlackGold_BB", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}BlackGold{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_BlackGold_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}BlackGold{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_BlackGold_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}BlackGold{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Creature_Barbwire_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Creature_Bars_Cutout", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_Creature_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Creature{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Blue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BASE" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Green", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BASE" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Orange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BASE" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Purple", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BASE" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Red", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BASE" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Yellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}BASE" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Cutout_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Cutout_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Cutout_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Cutout_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Cutout_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Cutout_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Cutout_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Cutout_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Cutout_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Cutout_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Cutout_K", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Cutout_L", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}CUTOUT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Paint_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}PAINT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Paint_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}PAINT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Paint_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}PAINT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Paint_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}PAINT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Paint_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}PAINT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Paint_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}PAINT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Paint_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}PAINT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Paint_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}PAINT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Paint_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}PAINT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Paint_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}PAINT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Design_Paint_K", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}PAINT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_SessionOG_Clean_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_SessionOG_Scratched_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_DGK_BackOf", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_DGK_Drippy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_DGK_Floating", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_DGK_General", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_DGK_GlowUp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_DGK_Guadalupe", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_DGK_MonoGram", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_DGK_Primo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_DGK_Rosary", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_DGK_Serape", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_DGK_SmileNow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_DGK_BackOf", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_DGK_Drippy", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_DGK_Floating", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_DGK_General", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_DGK_GlowUp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_DGK_Guadalupe", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_DGK_MonoGram", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_DGK_Primo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_DGK_Rosary", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_DGK_Serape", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_DGK_SmileNow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_DGK_Atlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_DGK_Atlas_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_DGK_Atlas_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}DGK{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Diz_SprayWings", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Diz_SprayWings", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_Diz_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Diz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Default_Black_Flypaper_Diecut", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Flypaper{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Flypaper_Swarm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Flypaper{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_Flypaper_Designs_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Flypaper{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Grizzly_GrizzlyTheBear", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Grizzly_GummyBears", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Grizzly_IceCreamTruck", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Grizzly_LapOfLuxury", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Grizzly_SittinOnChrome", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Grizzly_SnakeEyes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Grizzly_SplitStamped", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Grizzly_StageDive", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Grizzly_StampPrint", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Grizzly_TiffanyStamp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Grizzly_TigerKingOgBear", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_Grizzly_ToTheMax", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Grizzly_GrizzlyTheBear", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Grizzly_GummyBears", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Grizzly_IceCreamTruck", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Grizzly_LapOfLuxury", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Grizzly_SittinOnChrome", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Grizzly_SnakeEyes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Grizzly_SplitStamped", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Grizzly_StageDive", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Grizzly_StampPrint", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Grizzly_TiffanyStamp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Grizzly_TigerKingOgBear", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Grizzly_ToTheMax", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_Grizzly_Atlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_Grizzly_Atlas_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_Grizzly_Atlas_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Grizzly{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Magenta_Bear", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Magenta{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Magenta_Camera", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Magenta{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Magenta_Fly", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Magenta{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Magenta_Swans", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Magenta{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_Magenta_VX", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Magenta{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_Magenta_Atlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Magenta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_Magenta_Atlas_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}Magenta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_MOB_Palace", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_MOB_ArgyleBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_MOB_BigLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_MOB_Palace", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_MOB_PinkStripes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_MOB_Rainbow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_MOB_RainbowBigLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_MOB_SmallLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_MOB_Wallpaper", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_MOBa_Atlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_MOBa_Atlas_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}MOB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_NBS_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_NBS_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_NBS_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_NBS_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_NBS_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}NBS{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_NoComply_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_NoComply_Designs_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}NoComply{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_SM_ColinColn", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_SM_LesToasters", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_SM_Mesa", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_SM_ColinColn", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_SM_LesToasters", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_SM_Mesa", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_SM_Atlas", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}SponsorMe{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_TY_DaeLa", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_TY_ModernLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_TY_DaeLa", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_TY_ModernLogo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_ThankYou_Atlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}ThankYou{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_TMNT_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_TMNT_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_TMNT_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_GT_TMNT_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_TMNT_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_TMNT_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_TMNT_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_GT_TMNT_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GT_TMNT_Atlas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}GripTapes{Path.DirectorySeparatorChar}TMNT{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_SnowSkate_Deck", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}SnowSkate" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_SnowSkate", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}SnowSkate" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_HG_SnowSkate", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}SnowSkate" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_SnowSkate", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}SnowSkate" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "InvisiblePart_Snowskate", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}SnowSkate" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Layers", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}SnowSkate" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_SnowSkate_Deck", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}SnowSkate" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Snowskate_Wheel", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}SnowSkate" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_BasePlate_SnowSkate", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}SnowSkate" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_SnowSkate", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}SnowSkate" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Wood_Top", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}SnowSkate" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "WoodBottom", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}SnowSkate" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Destructo_D1_129_Low", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Dest_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Dest_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BasePlate_Destructo_Avalanche", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BasePlate_Destructo_D1Raw", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BasePlate_Destructo_D2_Blue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BasePlate_Destructo_Sendai_Blue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BasePlate_Destructo_Sendai_Mint", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BasePlate_Destructo_Sendai_Orange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Hanger_Destructo_Avalanche", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Hanger_Destructo_D1Raw", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Hanger_Destructo_D1Raw1", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Hanger_Destructo_D2_Blue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Hanger_Destructo_Sendai_Blue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Hanger_Destructo_Sendai_Mint", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Hanger_Destructo_Sendai_Orange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_BasePlate_Master_Inst", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MAI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Truck_BasePlate_Destructo_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Truck_BasePlate_Destructo_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Truck_BasePlate_Destructo_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Truck_Hanger_Destructo_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Truck_Hanger_Destructo_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_BasePlate_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MASTER" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Trucks_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}MASTER" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_GEN_BrushedMetal_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Hanger_Destructo_ColorMask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_DesturctoPrints", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_BasePlate_Destructo_ID_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_Destructo_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_Destructo_03_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_Destructo_04_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_Destructo_05_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Destructo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_GrindKing_Disruptor_139", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_GrindKing_Disruptor_144", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_GrindKing_Disruptor_149", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Truck_GrindKing_Disruptor_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Truck_GrindKing_Disruptor_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_Truck_GrindKing_Disruptor_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_GrindKing_Disruptor_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_GrindKing_Disruptor_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_GrindKing_Disruptor_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_GrindKing_Disruptor_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Truck_Hanger_GrindKing_Disruptor_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}GrindKing{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_BasePlate_KRUX_K5NoraTriangle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_KRUX_K5NoraTriangle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_KRUX_K5", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_KRUX_K5NoraTriangle", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_KRUX_K5_BasePlate", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_KRUX_K5_Hanger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_KRUX_K5NoraTriangle_BasePlate", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_KRUX_K5NoraTriangle_Hanger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_Tk_BP_KRUX_K5NoraTriangle_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_KRUX_K5NoraTriangle_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_KRUX_K5NoraTriangle_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_KRUX_K5NoraTriangle_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_KRUX_K5NoraTriangle_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Krux{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "SKXX_Truck_BasePlate_Tensor_MagLight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "SKXX_Truck_Hanger_Tensor_MagLight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Tensor_MagLIght_ElemetsDaewon", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Tensor_MagLIght_NewGeometry", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Tensor_MagLIght_NewGeometry_Gold", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Tensor_MagLIght_NewGeometry_Red", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Tensor_MagLIght_NewGeometry_Rose", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Tensor_MagLIght_ElemetsDaewon_BasePlate", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Tensor_MagLIght_ElemetsDaewon_Hanger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Tensor_MagLIght_NewGeometry_BasePlate", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Tensor_MagLIght_NewGeometry_Hanger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Tensor_MagLIght_NewGeometry_Hanger_Gold", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Tensor_MagLIght_NewGeometry_Hanger_Red", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_Tensor_MagLIght_NewGeometry_Hanger_RoseGold", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_Tensor_MagLight_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_Tensor_MagLight_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Tensor_MagLight_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Tensor_MagLight_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Tensor_MagLight_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Tensor{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "SKXX_Truck_BasePlate_Thunder", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "SKXX_Truck_Hanger_Thunder", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Thunder_AppleyardExperience", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Thunder_Brevard", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Thunder_ColeBouquetLights", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BP_Thunder_AppleyardExperience", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BP_Thunder_Brevard", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BP_Thunder_ColeBouquetLights", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_HAN_Thunder_AppleyardExperience", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_HAN_Thunder_Brevard", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_HAN_Thunder_ColeBouquetLights", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_Thunder_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_Thunder_Brevard_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_Thunder_Brevard_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_Thunder_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_Thunder_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Thunder_Appleyard_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Thunder_Brevard_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Thunder_Brevard_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Thunder_Cole_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Thunder_Mask", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Thunder_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Thunder{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "SKXX_Truck_BasePlate_Venture_VLight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Venture{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "SKXX_Truck_Hanger_Venture_VLight", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Venture{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_Venture_Pudwill", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Venture{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BP_Venture_Pudwill", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Venture{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_HG_Venture_Pudwill", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Venture{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_Venture_MASK", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Venture{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_BP_Venture_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Venture{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Venture_MASK", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Venture{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Venture_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Venture{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_TK_HG_Venture_Pudwill_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Venture{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_TK_BP_Default_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_Debug_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_Default_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_TK_BP_Default_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Baseplate{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_139_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_139_Mid_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_144_Mid_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_159_Mid_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_169_Mid_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_169_Mid_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Truck_Hanger_Default_8125_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Truck{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Hanger{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_Nexus_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_Nexus_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_Nexus_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_Nexus_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_Nexus_NeonBlue_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_Nexus_NeonGreen_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_Nexus_NeonPink_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_Nexus_NeonYellow_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_StreetSlims_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_StreetSlims_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_TorusUltra_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_TorusUltra_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABS_TorusUltra_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABX_Torus_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_ABX_Torus_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_DualUltra_Calssic_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_DualUltra_Calssic_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_DualUltra_Calssic_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_Evolution_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_Evolution_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_Evolution_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_GT1_WideBoy_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Autobahn_GT1_WideBoy_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_ABS_Nexus_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_ABS_Nexus_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_ABS_Nexus_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_ABS_NexusBlack_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_ABX_Tours_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_ABX_Tours_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_DualUltra_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_DualUltra_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_DualUltra_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_DualUltra_Classic_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_DualUltra_Classic_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_DualUltra_Classic_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_Evolution_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_Evolution_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_Evolution_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_GT1_Widebody_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_GT1_Widebody_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_Nexus_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_Nexus_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_Nexus_52B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_Nexus_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_Nexus_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_StreetSlims_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_StreetSlims_50_Metallic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_StreetSlims_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_StreetSlims_52_Metallic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_TorusUltra_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_TorusUltra_50_Metallic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_TorusUltra_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_TorusUltra_52_Metallic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_TorusUltra_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Autobahn_TorusUltra_54_Metallic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}AutobahnWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_BoardyCakes_42_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_BoardyCakes_BenchWarmers_48_5mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_BoardyCakes_DripDrops_39mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_BoardyCakes_FatBundts_44mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_BoardyCakes_OGs_42mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_BoardyCakes_PrincessCut_49mm", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_BoardyCakes_BenchWarmers_48_5", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_BoardyCakes_DripDrops_39", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_BoardyCakes_OGs_42", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_DF_BoardyCakes", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}BoardyCakes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Haze_101ChiChi_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_101PrimeCut_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_10thYear_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_DeathOnAcid_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_DeathOnAcid_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_DeathOnAcid_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_DeathOnAcid_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_Hazy_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_Lurk_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_Lurk_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_PrimeCut_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_PrimeCut_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_PrimeCut_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_Sneak_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Haze_Twirl_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_101ChiChi_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_101PrimeCut_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_10thYear_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_DeathOnAcid_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_DeathOnAcid_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_DeathOnAcid_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_DeathOnAcid_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_Hazy_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_Lurk_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_Lurk_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_PrimeCut_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_PrimeCut_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_PrimeCut_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_Sneak_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Haze_Twirl_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Haze{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_OJ_Erick_Winkowski_Tropics_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_OJ_JakeWootenScreamingCast_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_OJ_MiltonMartinezItsUsGold_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_OJ_NickBoserioLadybugs_60", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_OJ_TeamLineNOMAD_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_OJ_Erick_Winkowski_Tropics_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_OJ_JakeWootenScreamingCast_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_OJ_MiltonMartinezItsUsGold_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_OJ_NickBoserioLadybugs_60", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_OJ_NoraVasconcellosDreams_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_OJ_TeamLineNOMAD_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_Erick_Winkowski_Tropics_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_Erick_Winkowski_Tropics_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_JakeWootenScreamingCast_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_JakeWootenScreamingCast_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_MiltonMartinezItsUsGold_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_MiltonMartinezItsUsGold_A_Metall", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_MiltonMartinezItsUsGold_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_NickBoserioLadybugs_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_NickBoserioLadybugs_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_NoraVasconcellosDreams_54_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_NoraVasconcellosDreams_54_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_TeamLineNOMAD54_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_OJ_TeamLineNOMAD54_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}OJs{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Ricta_Clouds_Black_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Ricta_Clouds_Red_57", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Ricta_Clouds_White_60", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Ricta_Impeller_Brevard_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Ricta_Naturals_Vortex_DesArmo_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Ricta_Sparx_MainFrame_Black_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Ricta_Speedrings_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Clouds_Black_56_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Clouds_Black_56_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Clouds_NM", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Clouds_Red_57_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Clouds_Red_57_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Clouds_White_60_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Clouds_White_60_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Impeller_Brevard_54_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Impeller_Brevard_54_A_Metallic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Impeller_Brevard_54_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Naturals_Vortex_DesArmo_51_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Naturals_Vortex_DesArmo_51_A_Metallic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Naturals_Vortex_DesArmo_51_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Sparx_MainFrame_Black_53_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Sparx_MainFrame_Black_53_A_Metallic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Sparx_MainFrame_Black_53_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Sparx_MainFrame_Black_53_B_Metallic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Speedrings_52_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Speedrings_52_A_Metallic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Speedrings_52_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Ricta_Speedrings_52_B_Metallic", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Ricta{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Satori_BigLinkSeriesClassic51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Satori_LilNuggs_GooBall_54_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Satori_MeditationSeriesClassic_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Satori_RedEye_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Satori_DeLaSatori_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Satori_LinkedLogoTwoTone_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Satori_Paisley_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Satori_TopShelfElephant_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Satori_TopShelfElephant_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Satori_TopShelfElephant_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Satori_TopShelfElephant_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SatoriWheels_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SatoriWheels_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SatoriWheels_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SatoriWheels_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Satori_Concial_DelaSatori_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Satori_Concial_PaisleyLink_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Satori_Concial_TopShelfElephant_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Satori_Concial_TopShelfElephant_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Satori_Concial_TopShelfElephant_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Satori_Concial_TopShelfElephant_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Satori_LinkedLogoTwoTone_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SatoriWheels{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_AustynGilette_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_BackToThe90s_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_BubbsSeries_AustynGillette_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_BubbsSeries_MarkSuciu_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_ClassicSeries_AusytnsAlmonds_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_ClassicSeries_DannyGarica_PinaoHands_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_ClassicSeries_DrakeJones_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_ClassicSeries_RaymondMolinar_Polaroid_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_CoffeCruiser_50_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_CoffeCruiser_50_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_CoffeCruiser_50_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_CoffeCruiser_52_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_CoffeCruiser_52_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_CoffeCruiser_52_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_CoffeCruiser_54_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_CoffeCruiser_54_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_CoffeCruiser_54_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_CoffeCruiser_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_DelicateSeries_AustynGillette_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_DelicateSeries_MarkSuciu_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_DelicateSeries_SammyMontano_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_GroceryBag_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_GroceryBag_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_GroceryBag_53_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_GroceryBag_53_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_GroceryBag_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_GroceryBag_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_GroceryBag_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_MarkSuciu_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_MaxGeronzi_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_MikeArnold_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_PencilPushersSeries_AustynGillette_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_PencilPushersSeries_ChrisJones_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_PencilPushersSeries_JoshPall_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_PencilPushersSeries_TomKnox_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_ReallysmlWheels_40_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_ReallysmlWheels_40_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_SucculentCruiser_FireWater_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_SucculentCruiser_FireWater_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_Team_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_TheLoveSeries_MikeArnold_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_TheLoveSeries_SammyMontano_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_TheLoveSeries_TomKnox_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_Toonies_MikeArnold_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_Toonies_TomKnox_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_Toonies_YounessAmraniOG_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_SML_YounessAmrani_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_SML_JamesCraig_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_SML_SoldticeSeries_AustynGillette_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_SML_SoldticeSeries_TomKnox_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_SML_SoldticeSeries_YounessAmrani_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_04", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_05", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_06", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_07", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_08", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_09", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_10", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_11", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_12", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_13", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_14", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_15", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_16", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_17", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_18", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_19", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_20", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_21", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_22", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_23", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_24", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_25", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_26", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_27", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_28", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_29", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_30", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_31", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_32", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_33", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_34", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_35", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_36", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_37", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_38", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_39", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_40", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_41", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_42", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_43", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_44", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_SML_45", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_SML_ClassicSeries_DannyGarcia_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_SML_ClassicSeries_JamesCraig_OG_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_SML_GroceryBag_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_SML_GroceryBag_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_SML_SoldticeSeries_AustynGillette_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_SML_SoldticeSeries_TomKnox_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_SML_SoldticeSeries_YounessAmrani_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SML{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Snot_LilBoogers_48", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Snot_Team_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Snot_Team_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Snot_Team_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Snot_Team_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Snot_Team_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Snot_Team_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Snot_Team_58", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Snot_LilBoogers_48", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Snot_Team_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Snot_Team_52_101A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Snot_Team_53_90A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Snot_Team_54_101A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Snot_Team_54_97A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Snot_Team_55_99A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Snot_Team_56_101A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Snot_Team_56_97A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Snot_Team_58_99A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}SnotWheelCo{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_BigheadBurnPosse_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_BigheadPurpleYellow_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_BigheadWhiteGreen_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_BigheadWhitePurple_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_BigheadWhiteRed_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_ClassicBlue_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_ClassicGold_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_ClassicGray_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_ClassicGreen_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_ClassicOrange_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_ClassicPurple_58", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_ClassicRed_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_ClassicYellow_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_FormulaFour_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_FormulaFourEternals_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_NeonBigheadsBlue_57", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_NeonBigheadsGreen_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_NeonBigheadsPink_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CIT_WL_Spitfire_NeonBigheadsYellow_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}CIT" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_BigheadBurnPosse_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_BigheadPurpleYellow_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_BigheadWhiteGreen_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_BigheadWhitePurple_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_BigheadWhiteRed_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_ClassicBlue_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_ClassicGold_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_ClassicGray_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_ClassicGreen_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_ClassicOrange_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_ClassicPurple_58", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_ClassicRed_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_ClassicYellow_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_FormulaFour_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_FormulaFourEternals_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_NeonBigheadsBlue_57", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_NeonBigheadsGreen_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_NeonBigheadsPink_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Spitfire_NeonBigheadsYellow_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_BigheadBurnPosse_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_BigheadPurpleYellow_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_BigheadWhiteGreen_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_BigheadWhitePurple_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_BigheadWhiteRed_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_ClassicBlue_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_ClassicGold_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_ClassicGray_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_ClassicGreen_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_ClassicOrange_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_ClassicPurple_58", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_ClassicRed_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_ClassicYellow_55", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_FormulaFour_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_FormulaFourEternals_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_NeonBigheadsBlue_57", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_NeonBigheadsGreen_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_NeonBigheadsPink_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Spitfire_NeonBigheadsYellow_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Brands{Path.DirectorySeparatorChar}Spitfire{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassic_Default_46_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassic_Default_50_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassic_Default_51_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassic_Default_53_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassic_Default_55_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_Classic_Default_56_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_Classic_Default_58_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassicWide_Default_48_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassicWide_Default_50_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassicWide_Default_51_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassicWide_Default_52_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassicWide_Default_53_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_CLassicWide_Default_54_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_ConicWide_Default_50_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_ConicWide_Default_51_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_ConicWide_Default_52_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_ConicWide_Default_53_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_ConicWide_Default_54_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_ConicWide_Default_56_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_Default_RadialWide_46_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_Default_RadialWide_48_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_Default_RadialWide_50_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_Default_RadialWide_52_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_Default_RadialWide_54_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "STM_SKXX_Wheel_Default_RadialWide_56_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Basemesh" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_32", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_36", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_46", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_ConicWide_48", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_ConicWide_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_ConicWide_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_ConicWide_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_ConicWide_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_ConicWide_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_ConicWide_56", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_50_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_50_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_50_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_50_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_Turtoise_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_Turtoise_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_Turtoise_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_Turtoise_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_Turtoise_Wide_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_Turtoise_Wide_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_Turtoise_Wide_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_Turtoise_Wide_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_Wide_50_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_Wide_50_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_Wide_50_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Trans_Wide_50_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Wide_48", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Wide_50", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Wide_51", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Wide_52", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Wide_53", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Defualt_Wide_54", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Transparent_02", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_Wheel_Transparent_03", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WL_Default_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_DF_Default_01", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DEBUG_WheelPlacement", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_32mm_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_36mm_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_50mm_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_50mm_BC_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_50mm_BC_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_50mm_BC_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_50mm_BC_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_50mm_Don_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_50mm_Leo_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_50mm_Michel_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_50mm_Raph_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_51mm_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_52mm_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_53mm_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_54mm_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DF_Default_56mm_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Default_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Default_01_MG", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Default_02_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Default_Blank_01_BC", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WL_Default_Dirt_01_Alpha", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_WheelOpaqueDualArt_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wheel{Path.DirectorySeparatorChar}Material" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Black", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Blue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Blue_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Blue_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Blue_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Brown", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Brown_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Brown_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Brown_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Green", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Green_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Green_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Magenta", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Pink", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Pink_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Purple_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Purple_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Purple_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Red_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Red_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Teal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Teal_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Teal_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Teal_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Default_Yellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_Grey", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_White", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodBottom" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Blue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Blue_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Brown", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Green_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Orange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Pink", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Purple", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Purple_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Red_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Teal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_White", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_WhitePaint", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Yellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Default_Yellow_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Materials{Path.DirectorySeparatorChar}WoodTop" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WD_Default_Scratch_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Default{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_WG_Deviant_RibsManFiendClub", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Deviant" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Element_Landrein", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Element_MoonDust", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Element_Square30", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Element_Atlas", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Element{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_BubbleLogoBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_BubbleLogoOrange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_BubbleLogoOrange_Orange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_BubbleLogoPink_Orange", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_HDK", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_OdsseyStencil_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_OdsseyStencil_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_QuattroOdysseyLumo_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_QuattroOdysseyLumo_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_ScriptBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_ScriptBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_TeamSorryBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_TeamSorryYellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Flip_TeamStrobe", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Flip_Atlas", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Flip{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Jacuzzi_500Years", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Jacuzzi_BunnyLamp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Jacuzzi_Flavour", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Jacuzzi_500Years", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Jacuzzi_BunnyLamp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Jacuzzi_Flavour", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WT_Jacuzzi_500Years", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WT_Jacuzzi_BunnyLamp", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WT_Jacuzzi_Flavour", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Jacuzzi{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Madrid_ExplosionBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Madrid_ExplosionRed", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Madrid_ExplosionWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Madrid_ExplosionYellow", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Madrid_Logo", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Madrid_TaroCardSeriesas_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Madrid_TaroCardSeriesas_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_Madrid_WingsBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WT_Madrid_Atlas", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Madrid{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_PlanB_LewBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_PlanB_LewBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_PlanB_LewTeal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_PlanB_LewWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WT_PlanB_LewBlack", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WT_PlanB_LewBlue", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WT_PlanB_LewTeal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WT_PlanB_LewWhite", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}PlanB{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WB_SandlotTimes_HoppingFences", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SandlotTimes_HoppingFences_WoodTint", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SandlotTimes{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_SantaCruz_ClassicDot", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_SantaCruz_DressnRoses", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_SantaCruz_KendallEndOfTheWorld", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_SantaCruz_KnoxFirepit", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_SantaCruz_MeekOGSlasher", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_SantaCruz_SalbaTiger", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_SantaCruz_ScreamingHand", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SantaCruz_KnoxFirepit", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_DG_SantaCruz_MeekOGSlasher", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WT_SantaCruz_Atlas", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WT_SantaCruz_ScreamingHand_8250", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}SantaCruz{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_WG_Welcome_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_WG_Welcome_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_WG_Welcome_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_WG_Welcome_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_WG_Welcome_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_WG_Welcome_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_WG_Welcome_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_WG_Welcome_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_WG_Welcome_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_WG_Welcome_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAI_WT_WG_Welcome_K", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_A", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_B", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_C", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_D", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_D_Metal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_E", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_F", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_G", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_H", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_I", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_J", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_J_Metal", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "TEX_WG_Welcome_K", RelativePath = $"Customization{Path.DirectorySeparatorChar}Skateboards{Path.DirectorySeparatorChar}Wood{Path.DirectorySeparatorChar}Welcome{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "MAT_NewItem_Master", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI{Path.DirectorySeparatorChar}Materials" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "PBP_InputController_Widget", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "CRT_CustomizationCharacter", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "GridItem_ShirtImage", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "GridItem_ShoesImage", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI{Path.DirectorySeparatorChar}Textures" });
            _texturePaths.Add(new TexturePathInfo() { TextureName = "Mat_RenderTarget", RelativePath = $"Customization{Path.DirectorySeparatorChar}UI{Path.DirectorySeparatorChar}Textures" });

            return _texturePaths;
        }
    }
}
